/*

Copyright (C) 2018 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include<iostream>
#include<iomanip>
#include"../movegen/move.h"
#include"../search/multipv.h"
using namespace std;

//-------------------------------------------------------
//
//           unittest multipv:
//

void test_multipv()
{
  cout << "Test MultiPV .........................................." << flush;
  bool r=true;

  PrincipalVariation a,b,c;
  a[0]=Move(e2,e4,0,WhitePawn(),NoPiece());
  a[1]=Move(e7,e5,0,BlackPawn(),NoPiece());
  a.set_score(100);
    
  b[0]=Move(d2,d4,0,WhitePawn(),NoPiece());
  b[1]=Move(d7,d5,0,BlackPawn(),NoPiece());
  b.set_score(200);

  c[0]=Move(g1,f3,0,WhiteKnight(),NoPiece());
  c[1]=Move(g8,f6,0,BlackKnight(),NoPiece());
  c.set_score(300);

  {
    MultiPV mpv;
    mpv.set_number(3);
    mpv.add(a);
    mpv.add(b);
    mpv.add(c);
    if(mpv.number()!=3){r=false;cerr << "error1" << endl;}
    if(mpv.begin()->score()!=300){r=false;cerr << "error2" << endl;}
    if((mpv.end()-1)->score()!=100){r=false;cerr << "error3" << endl;}
  }

  {
    MultiPV mpv;
    mpv.set_number(3);
    mpv.add(c);
    mpv.add(c);
    mpv.add(c);
    mpv.add(b);
    mpv.add(b);
    mpv.add(b);
    mpv.add(a);
    mpv.add(a);
    mpv.add(a);
    if(mpv[0].score()!=300){r=false;cerr << "error4" << endl;}
    if(mpv[mpv.size()-1].score()!=100){r=false;cerr << "error5" << endl;}
  }

  //---------------------- test randomize: ----------------
  {
    MultiPV mpv;
    mpv.set_number(3);
    mpv.add(a);
    mpv.add(b);
    mpv.add(c);
    
    int i{0};
    while(i<50){
      mpv.randomize(300);
      if(mpv[0].score()==100){break;}
      i++;
    }
    if(i>40){r=false;}
  }
  
  {
    MultiPV mpv;
    mpv.set_number(3);
    mpv.add(a);
    mpv.add(b);
    mpv.add(c);
    
    int i{0};
    while(i<50){
      mpv.randomize(150);
      if(mpv[0].score()==100){break;}
      i++;
    }
    if(i<50){
      cout << "i=" << i << endl;
      cout << mpv[0].score() << " " << mpv[1].score() << " " << mpv[2].score() << endl; 
      r=false;
    }
  }
  //-------------------------------------------------------
  

  if(r){cout << "OK" << endl;}else{cout << "FAIL" << endl;}

}






