/*

Copyright (C) 2018 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include<iostream>
#include<iomanip>
#include"../board/cposition.h"
#include"../hashtable/transposition.h"
#include"../board/hasher.h"
using namespace std;

//-------------------------------------------------------
//
//           unittest TranspositionTable
//


void test_transposition()
{
  bool r=true;
  cout << "Test TranspositionTable() ............................." << flush;


  Move m;m.from()=e2;m.to()=e4;
  ChessPosition cp;
  cp.newgame();
  Hasher hh;hh.init(cp);
  TranspositionTable tt;
 
  {
    TTrecord ttr;
    ttr.set_score(25,0);
    ttr.set_bound(exact_value);
    ttr.set_draft(4);
    ttr.set_move(m);
    ttr.set_key(hh.hash());
    tt.write(ttr);
    TTrecord ttr2;
    ttr2.set_key(hh.hash());
    if(!tt.read(ttr2)){r=false;cout << "error(1)" << endl;}

    if(ttr!=ttr2){r=false;cout << "error(2)" << endl;}
    
    NullMove m0;
    cp.make_move(m0);
    long long h=cp.get_hash();
    ttr2.set_key(h);
    if(tt.read(ttr2)){r=false;cout << "error(3)" << endl;}
  }

  tt.set_size(70);
  if(tt.get_size()!=64){r=false;cout << "error(4)" << endl;}
  tt.set_size(128);
  if(tt.get_size()!=128){r=false;cout << "error(5)" << endl;}
  tt.clear();
  
  {
    int key=10000;
    TTrecord ttr;
    ttr.set_score(25,0);
    ttr.set_bound(exact_value);
    ttr.set_draft(4);
    ttr.set_move(m);
    ttr.set_key(key);
    tt.write(ttr);

    tt.set_threshold(key*64);
    if(tt.read(ttr)){r=false;cout << "error(6)" << endl;}
    ttr.set_score(100,0);
    tt.write(ttr);
    tt.set_threshold(0);
    if(!tt.read(ttr)){r=false;cout << "error(7)" << endl;}
    if(ttr.get_score(0)!=25){r=false;cout << "error(8)" << endl;}


    tt.set_threshold(5000000);
    if(tt.get_threshold()!=5000000){r=false;cout << "error(9)" << endl;}
  }

  

  if(r){cout << "OK" << endl;}else{cout << "FAIL" << endl;}

}






