/*

Copyright (C) 2018 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include<iostream>
#include<iomanip>
#include<fstream>
#include<random>
#include<cassert>
#include"../movegen/move.h"
#include"hasher.h"
#include"cposition.h"
#include"trandom.h"
using std::rand;
using std::cout;
using std::endl;
using std::hex;
using std::dec;
using std::ofstream;
using std::ios_base;


namespace PolyglotPiece 
{
  constexpr int black_pawn=0,white_pawn=1,black_knight=2,white_knight=3,
    black_bishop=4,white_bishop=5,black_rook=6,white_rook=7,
    black_queen=8,white_queen=9,black_king=10,white_king=11;
}

Hasher::Hasher()
{
  m_hash.clear();m_hash.push_back(0);
  m_pawnhash.clear();m_pawnhash.push_back(0);
  int i=0;
  while(i<64)
    {
      m_piece_square[no_piece_id*64+i]=0;
      m_piece_square[white_pawn_id*64+i]=rndh[PolyglotPiece::white_pawn*64+i];
      m_piece_square[white_knight_id*64+i]=rndh[PolyglotPiece::white_knight*64+i];
      m_piece_square[white_bishop_id*64+i]=rndh[PolyglotPiece::white_bishop*64+i];
      m_piece_square[white_rook_id*64+i]=rndh[PolyglotPiece::white_rook*64+i];
      m_piece_square[white_queen_id*64+i]=rndh[PolyglotPiece::white_queen*64+i];
      m_piece_square[white_king_id*64+i]=rndh[PolyglotPiece::white_king*64+i];
      m_piece_square[black_pawn_id*64+i]=rndh[PolyglotPiece::black_pawn*64+i];
      m_piece_square[black_knight_id*64+i]=rndh[PolyglotPiece::black_knight*64+i];
      m_piece_square[black_bishop_id*64+i]=rndh[PolyglotPiece::black_bishop*64+i];
      m_piece_square[black_rook_id*64+i]=rndh[PolyglotPiece::black_rook*64+i];
      m_piece_square[black_queen_id*64+i]=rndh[PolyglotPiece::black_queen*64+i];
      m_piece_square[black_king_id*64+i]=rndh[PolyglotPiece::black_king*64+i];
      i++;
    }

  i=0;
  while(i<64)
    {
      m_pawn_square[no_piece_id*64+i]=0;
      m_pawn_square[white_pawn_id*64+i]=rndh[PolyglotPiece::white_pawn*64+i];
      m_pawn_square[white_knight_id*64+i]=0;
      m_pawn_square[white_bishop_id*64+i]=0;
      m_pawn_square[white_rook_id*64+i]=0;
      m_pawn_square[white_queen_id*64+i]=0;
      m_pawn_square[white_king_id*64+i]=0;
      m_pawn_square[black_pawn_id*64+i]=rndh[PolyglotPiece::black_pawn*64+i];
      m_pawn_square[black_knight_id*64+i]=0;
      m_pawn_square[black_bishop_id*64+i]=0;
      m_pawn_square[black_rook_id*64+i]=0;
      m_pawn_square[black_queen_id*64+i]=0;
      m_pawn_square[black_king_id*64+i]=0;
      i++;
    }

  i=768;
  m_castle_K=rndh[i++];
  m_castle_Q=rndh[i++];
  m_castle_k=rndh[i++];
  m_castle_q=rndh[i++];

  for(int j=0;j<8;j++)
    {
      m_en_passant[j]=rndh[i++];
    }
  m_turn=rndh[i];
}


void Hasher::init(const ChessPosition& x)
{
  cp=&x;
  init(cp);
}

void Hasher::init(const ChessPosition* x)
{
  BitBoard bb_sq,p;
  Square sq;
  cp=x;
  //Piece p;
  m_hash.clear();m_hash.push_back(0);
  m_pawnhash.clear();m_pawnhash.push_back(0);

  p=cp->white_pawn();
  while(p){
    bb_sq=p.get_reset_square();
    sq=bb_sq.get_first_square();
    m_hash.back()^=m_piece_square[white_pawn_id*64+sq.id()];
    m_pawnhash.back()^=m_piece_square[white_pawn_id*64+sq.id()];
  }
  p=cp->white_knight();
  while(p){
    bb_sq=p.get_reset_square();
    sq=bb_sq.get_first_square();
    m_hash.back()^=m_piece_square[white_knight_id*64+sq.id()];
  }
  p=cp->white_bishop();
  while(p){
    bb_sq=p.get_reset_square();
    sq=bb_sq.get_first_square();
    m_hash.back()^=m_piece_square[white_bishop_id*64+sq.id()];
  }
  p=cp->white_rook();
  while(p){
    bb_sq=p.get_reset_square();
    sq=bb_sq.get_first_square();
    m_hash.back()^=m_piece_square[white_rook_id*64+sq.id()];
  }
  p=cp->white_queen();
  while(p){
    bb_sq=p.get_reset_square();
    sq=bb_sq.get_first_square();
    m_hash.back()^=m_piece_square[white_queen_id*64+sq.id()];
  }
  p=cp->white_king();
  while(p){
    bb_sq=p.get_reset_square();
    sq=bb_sq.get_first_square();
    m_hash.back()^=m_piece_square[white_king_id*64+sq.id()];
  }

  p=cp->black_pawn();
  while(p){
    bb_sq=p.get_reset_square();
    sq=bb_sq.get_first_square();
    m_hash.back()^=m_piece_square[black_pawn_id*64+sq.id()];
    m_pawnhash.back()^=m_piece_square[black_pawn_id*64+sq.id()];
  }
  p=cp->black_knight();
  while(p){
    bb_sq=p.get_reset_square();
    sq=bb_sq.get_first_square();
    m_hash.back()^=m_piece_square[black_knight_id*64+sq.id()];
  }
  p=cp->black_bishop();
  while(p){
    bb_sq=p.get_reset_square();
    sq=bb_sq.get_first_square();
    m_hash.back()^=m_piece_square[black_bishop_id*64+sq.id()];
  }
  p=cp->black_rook();
  while(p){
    bb_sq=p.get_reset_square();
    sq=bb_sq.get_first_square();
    m_hash.back()^=m_piece_square[black_rook_id*64+sq.id()];
  }
  p=cp->black_queen();
  while(p){
    bb_sq=p.get_reset_square();
    sq=bb_sq.get_first_square();
    m_hash.back()^=m_piece_square[black_queen_id*64+sq.id()];
  }
  p=cp->black_king();
  while(p){
    bb_sq=p.get_reset_square();
    sq=bb_sq.get_first_square();
    m_hash.back()^=m_piece_square[black_king_id*64+sq.id()];
  }


}

void Hasher::make_move(const Move& m)
{
  //Move m{mm};
  m_hash.push_back(m_hash.back());
  m_pawnhash.push_back(m_pawnhash.back());
  switch(m.code())
    {
    case normal_move:
      m_hash.back()^=m_piece_square[m.piece_id()*64+m.from().id()];
      m_hash.back()^=m_piece_square[m.piece_id()*64+m.to().id()];
      m_hash.back()^=m_piece_square[m.capture_id()*64+m.to().id()];
      m_pawnhash.back()^=m_pawn_square[m.piece_id()*64+m.from().id()];
      m_pawnhash.back()^=m_pawn_square[m.piece_id()*64+m.to().id()];
      m_pawnhash.back()^=m_pawn_square[m.capture_id()*64+m.to().id()];
      break;
    case promo_queen:
      if(m.to()>m.from())
	{
	  m_hash.back()^=m_piece_square[white_pawn_id*64+m.from().id()];
	  m_pawnhash.back()^=m_piece_square[white_pawn_id*64+m.from().id()];
	  m_hash.back()^=m_piece_square[white_queen_id*64+m.to().id()];
	  m_hash.back()^=m_piece_square[m.capture_id()*64+m.to().id()];
	}
      else
	{
	  m_hash.back()^=m_piece_square[black_pawn_id*64+m.from().id()];
	  m_pawnhash.back()^=m_piece_square[black_pawn_id*64+m.from().id()];
	  m_hash.back()^=m_piece_square[black_queen_id*64+m.to().id()];
	  m_hash.back()^=m_piece_square[m.capture_id()*64+m.to().id()];
	}      
      break;
    case promo_rook:
      if(m.to()>m.from())
	{
	  m_hash.back()^=m_piece_square[white_pawn_id*64+m.from().id()];
	  m_pawnhash.back()^=m_piece_square[white_pawn_id*64+m.from().id()];
	  m_hash.back()^=m_piece_square[white_rook_id*64+m.to().id()];
	  m_hash.back()^=m_piece_square[m.capture_id()*64+m.to().id()];
	}
      else
	{
	  m_hash.back()^=m_piece_square[black_pawn_id*64+m.from().id()];
	  m_pawnhash.back()^=m_piece_square[black_pawn_id*64+m.from().id()];
	  m_hash.back()^=m_piece_square[black_rook_id*64+m.to().id()];
	  m_hash.back()^=m_piece_square[m.capture_id()*64+m.to().id()];
	}      
      break;
    case promo_bishop:
      if(m.to()>m.from())
	{
	  m_hash.back()^=m_piece_square[white_pawn_id*64+m.from().id()];
	  m_pawnhash.back()^=m_piece_square[white_pawn_id*64+m.from().id()];
	  m_hash.back()^=m_piece_square[white_bishop_id*64+m.to().id()];
	  m_hash.back()^=m_piece_square[m.capture_id()*64+m.to().id()];
	}
      else
	{
	  m_hash.back()^=m_piece_square[black_pawn_id*64+m.from().id()];
	  m_pawnhash.back()^=m_piece_square[black_pawn_id*64+m.from().id()];
	  m_hash.back()^=m_piece_square[black_bishop_id*64+m.to().id()];
	  m_hash.back()^=m_piece_square[m.capture_id()*64+m.to().id()];
	}      
      break;
    case promo_knight:
      if(m.to()>m.from())
	{
	  m_hash.back()^=m_piece_square[white_pawn_id*64+m.from().id()];
	  m_pawnhash.back()^=m_piece_square[white_pawn_id*64+m.from().id()];
	  m_hash.back()^=m_piece_square[white_knight_id*64+m.to().id()];
	  m_hash.back()^=m_piece_square[m.capture_id()*64+m.to().id()];
	}
      else
	{
	  m_hash.back()^=m_piece_square[black_pawn_id*64+m.from().id()];
	  m_pawnhash.back()^=m_piece_square[black_pawn_id*64+m.from().id()];
	  m_hash.back()^=m_piece_square[black_knight_id*64+m.to().id()];
	  m_hash.back()^=m_piece_square[m.capture_id()*64+m.to().id()];
	}      
      break;
    case castling:
      if(m.piece().white_king()){
	m_hash.back()^=m_piece_square[white_king_id*64+m.from().id()];
	m_hash.back()^=m_piece_square[white_rook_id*64+m.to().id()];
	if(m.from()<m.to()){
	  m_hash.back()^=m_piece_square[white_king_id*64+g1.id()];
	  m_hash.back()^=m_piece_square[white_rook_id*64+f1.id()];
	}else{
	  m_hash.back()^=m_piece_square[white_king_id*64+c1.id()];
	  m_hash.back()^=m_piece_square[white_rook_id*64+d1.id()];
	}
      }else{
	m_hash.back()^=m_piece_square[black_king_id*64+m.from().id()];
	m_hash.back()^=m_piece_square[black_rook_id*64+m.to().id()];
	if(m.from()<m.to()){
	  m_hash.back()^=m_piece_square[black_king_id*64+g8.id()];
	  m_hash.back()^=m_piece_square[black_rook_id*64+f8.id()];
	}else{
	  m_hash.back()^=m_piece_square[black_king_id*64+c8.id()];
	  m_hash.back()^=m_piece_square[black_rook_id*64+d8.id()];
	}
      }
      break;
    case en_passant:
      m_hash.back()^=m_piece_square[m.piece_id()*64+m.from().id()];
      m_hash.back()^=m_piece_square[m.piece_id()*64+m.to().id()];
      m_pawnhash.back()^=m_pawn_square[m.piece_id()*64+m.from().id()];
      m_pawnhash.back()^=m_pawn_square[m.piece_id()*64+m.to().id()];
      if(m.to()>m.from()){
	m_hash.back()^=m_piece_square[black_pawn_id*64+m.to().id()-8];
	m_pawnhash.back()^=m_pawn_square[black_pawn_id*64+m.to().id()-8];
      }else{
	m_hash.back()^=m_piece_square[white_pawn_id*64+m.to().id()+8];
	m_pawnhash.back()^=m_pawn_square[white_pawn_id*64+m.to().id()+8];
      }
      break;
    case nullmove:
      break;
    }
}


void Hasher::unmake_move(const Move& m)
{
  m_hash.pop_back();
  m_pawnhash.pop_back();
}

uint64_t Hasher::castle()const
{
  constexpr short castle_K=0x0080;
  constexpr short castle_Q=0x0001;
  constexpr short castle_k=0x8000;
  constexpr short castle_q=0x0100;
  uint64_t r{0};
  int c=cp->castlingrights();
  if(c==0)return 0;
  if(c&castle_Q){r^=m_castle_Q;}
  if(c&castle_K){r^=m_castle_K;}
  if(c&castle_q){r^=m_castle_q;}
  if(c&castle_k){r^=m_castle_k;}
  if(c & 0x7E7E){r^=static_cast<uint64_t>(c);}   // only for chess960
  
  return r;
}

uint64_t Hasher::hash()const
{
  uint64_t r=m_hash.back();
  r^=castle();
  if(cp->ep()!=no_square){
    r^=m_en_passant[(cp->ep()).file_number()];
  }
  r^=m_turn*cp->stm();
  return r;
}

uint64_t Hasher::pawnhash()const
{
  return m_pawnhash.back();
}

