/*

Copyright (C) 2018 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include"bkmove.h"


BKmove::BKmove() : m(0)
{
}

BKmove::BKmove(const Move& x)
{
  m=0;
  char to=x.to().id();
  switch(x.code())
    {
    case normal_move:break;
    case promo_queen : m=4;m<<=12;break;
    case promo_rook : m=3;m<<=12;break;
    case promo_bishop : m=2;m<<=12;break;
    case promo_knight : m=1;m<<=12;break;
    case castling:break;
    case en_passant:break;
    case nullmove:break;
    }
  m|=x.from().id()<<6;
  m|=to;
}

Move BKmove::operator()(const ChessPosition& cp)const
{
  if(nomove()){return Move();}
  Square f=from().id();
  Square t=to().id();
  char c=code();
  switch(c)
    {
    case 1:c=promo_knight;break;
    case 2:c=promo_bishop;break;
    case 3:c=promo_rook;break;
    case 4:c=promo_queen;break;
    default: c=normal_move;break;
    }
  if(cp.get_piece_from(from()).white_king() && cp.get_piece_from(to()).white_rook()){c=castling;}
  if(cp.get_piece_from(from()).black_king() && cp.get_piece_from(to()).black_rook()){c=castling;}

  //------------------------------------------
  Move m=cp.convert_to_move(f,t,static_cast<move_t>(c));
  //------------------------------------------
  
  return m;
}


bool BKmove::nomove()const
{
  if(m==0){return true;}else{return false;}
}

Square BKmove::from()const
{
  return Square(m>>6 & 0x3F);
}

Square BKmove::to()const
{
  return Square(m & 0x3F);
}

char BKmove::code()const
{
  return char(m>>12 & 0x07);
}

void BKmove::set_to(Square x)
{
  m&=0xFFC0;m|=x.id();
}

void BKmove::set_from(Square x)
{
  m&=0xF03F;m|=x.id()<<6;
}

void BKmove::set_code(char x)
{
  unsigned int ix=x;
  m&=0x0FFF;m|=(ix<<12);
}

void BKmove::clear()
{
  m=0;
}

unsigned short int BKmove::dump()const
{
  return m;
}
