/*

Copyright (C) 2018 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include<iostream>
#include<iomanip>
#include<sstream>
#include<stdexcept>
#include"book.h"
using std::cout;
using std::endl;
using std::hex;
using std::dec;
using std::logic_error;


Book::Book()
{
  m_bookname.clear();
  m_tm.start();
}

Book::Book(string s)
{
  m_bookname=s;
  m_ifs.open(m_bookname);
  if(!m_ifs.good()){string E="Book::open() open "+ m_bookname + " failed (1)";throw logic_error(E);}
  m_tm.start();
}

Book::~Book()
{
  if(m_ifs.is_open()){m_ifs.close();}
}

bool Book::is_open()const
{
  if(m_ifs.is_open()){return true;}else{return false;}
}

void Book::open()
{
  if(m_ifs.is_open()){m_ifs.close();}
  m_ifs.open(m_bookname);
  if(!m_ifs.good()){string E="Book::open() open "+ m_bookname + " failed (1)";throw logic_error(E);}
  m_ifs.seekg(0,ios_base::end);
  m_end=m_ifs.tellg();
  m_end/=16;
  m_end*=16;
}

void Book::close()
{
  m_ifs.close();
}

void Book::set_name(string s)
{
  m_bookname=s;
}

string Book::get_name()const
{
  return m_bookname;
}

BKmove Book::get_move(unsigned long long k)
{
  char buffer[16];
  vector<Book_record> record;

  int64_t posmin{0},posmax{m_end},m;
  while(posmin<posmax){
    m=(posmin+posmax)/2;
    m/=16;
    m*=16;
    if(posmin==m){break;}
    m_ifs.seekg(m);
    m_ifs.read(buffer,16);
    Book_record bkrec(buffer);
    if(bkrec.get_key()<k){
      posmin=m;
    }else if(bkrec.get_key()>k){
      posmax=m;
    }else{break;}
  }
  
  m_ifs.seekg(posmin);
  while(m_ifs.tellg()!=m_end){
    {
      m_ifs.read(buffer,16);
      if(m_ifs.tellg()==-1)break;
      Book_record bkrec(buffer);
      if(bkrec.get_key()==k){
	record.push_back(bkrec);
      }else if(bkrec.get_key()>k){
	break;
      }
    }
  }

  BKmove r;
  if(!record.empty()){
    int total_weight{0};
    for(auto rec : record){total_weight+=rec.get_weight();}
    
    int rnd_number=m_tm.total_time().nanoseconds();
    rnd_number%=total_weight;
    for(unsigned int i=0;i<record.size();i++)
      {
	rnd_number-=record[i].get_weight();
	if(rnd_number<=0){r=record[i].get_move();break;}
      }
  }
  
  return r;
}
