/*

Copyright (C) 2018 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#ifndef CONNECTIONS_H
#define CONNECTIONS_H

#include"square.h"
#include"../common/bitboard.h"
#include<vector>
using std::vector;



class Connections
{

 public:
  Connections();

  const BitBoard& w_pawn_attacks(const Square&)const;
  const BitBoard& b_pawn_attacks(const Square&)const;
  const BitBoard& knight_attacks(const Square&)const;
  const BitBoard& bishop_attacks(const Square&)const;
  const BitBoard& rook_attacks(const Square&)const;
  const BitBoard& queen_attacks(const Square&)const;
  const BitBoard& king_attacks(const Square&)const;

  const BitBoard& attacks_00(const Square&)const;
  const BitBoard& attacks_45(const Square&)const;
  const BitBoard& attacks_90(const Square&)const;
  const BitBoard& attacks_135(const Square&)const;
  const BitBoard& attacks_180(const Square&)const;
  const BitBoard& attacks_225(const Square&)const;
  const BitBoard& attacks_270(const Square&)const;
  const BitBoard& attacks_315(const Square&)const;

  const BitBoard& file(const Square&)const;
  const BitBoard& rank(const Square&)const;
  const BitBoard& ray(const Square&,const Square&)const;
  const BitBoard& line_segment(const Square&,const Square&)const;
  const BitBoard& closed_line_segment(const Square&,const Square&)const;


 private:
  vector<BitBoard> m_square;

  vector<BitBoard> mw_pawn_attacks;
  vector<BitBoard> mb_pawn_attacks;
  vector<BitBoard> mknight_attacks;
  vector<BitBoard> mbishop_attacks;
  vector<BitBoard> mrook_attacks;
  vector<BitBoard> mqueen_attacks;
  vector<BitBoard> mking_attacks;

  vector<BitBoard> m_attacks_00;
  vector<BitBoard> m_attacks_45;
  vector<BitBoard> m_attacks_90;
  vector<BitBoard> m_attacks_135;
  vector<BitBoard> m_attacks_180;
  vector<BitBoard> m_attacks_225;
  vector<BitBoard> m_attacks_270;
  vector<BitBoard> m_attacks_315;

  vector<BitBoard> m_file;
  vector<BitBoard> m_rank;

  vector<vector<BitBoard>> m_ray;
  vector<vector<BitBoard>> m_line_segment;
  vector<vector<BitBoard>> m_closed_line_segment;
  vector<vector<char>> m_attack_lines;

  void init_squares();

  void init_w_pawn_attacks();
  void init_b_pawn_attacks();
  void init_knight_attacks();
  void init_bishop_attacks();
  void init_rook_attacks();
  void init_queen_attacks();
  void init_king_attacks();

  void init_attacks_00();
  void init_attacks_45();
  void init_attacks_90();
  void init_attacks_135();
  void init_attacks_180();
  void init_attacks_225();
  void init_attacks_270();
  void init_attacks_315();

  void init_file();
  void init_rank();
  void init_ray();
  void init_linesegments();

};



#endif
