/*

Copyright (C) 2018 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include<iostream>
#include<iomanip>
#include<stdexcept>
#include<cmath>
#include<cassert>
#include"pst.h"
#include"../board/cposition.h"
#include"../common/numerals.h"
using std::logic_error;
using std::cout;
using std::endl;
using std::round;
using std::hex;
using std::dec;



const array<int,64> PieceSquareTables::pawn_mg
{
// A1                                                H1
  -52,   -52,   -52,   -52,   -52,   -52,   -52,   -52, 
  -27,    -2,    11,    24,    24,    11,    -2,   -27, 
  -27,    -2,    62,    75,    75,    62,    -2,   -27, 
  -14,    24,    87,   100,   100,    87,    24,   -14, 
  -14,    24,    37,    49,    49,    37,    24,   -14, 
  -14,    11,    24,    24,    24,    24,    11,   -14, 
  -27,    -2,    -2,    -2,    -2,    -2,    -2,   -27, 
  -52,   -52,   -52,   -52,   -52,   -52,   -52,   -52, 
// A8                                                H8
};

const array<int,64> PieceSquareTables::pawn_eg
{
// A1                                                H1
  -78,   -78,   -78,   -78,   -78,   -78,   -78,   -78, 
  -48,   -48,   -48,   -48,   -48,   -48,   -48,   -48, 
  -19,   -19,   -19,   -19,   -19,   -19,   -19,   -19, 
   11,    11,    11,    11,    11,    11,    11,    11, 
   41,    41,    41,    41,    41,    41,    41,    41, 
   70,    70,    70,    70,    70,    70,    70,    70, 
  100,   100,   100,   100,   100,   100,   100,   100, 
  -78,   -78,   -78,   -78,   -78,   -78,   -78,   -78, 
// A8                                                H8
};

const array<int,64> PieceSquareTables::knight_mg
{
// A1                                                H1
  -94,   -78,   -62,   -45,   -45,   -62,   -78,   -94, 
  -78,   -62,    -5,    11,    11,    -5,   -62,   -78, 
  -54,    11,    68,    60,    60,    68,    11,   -54, 
  -29,    35,    76,    68,    68,    76,    35,   -29, 
  -29,    35,    76,    68,    68,    76,    35,   -29, 
  -37,    27,   100,    92,    92,   100,    27,   -37, 
  -54,   -29,    43,    60,    60,    43,   -29,   -54, 
  -78,   -54,   -29,   -13,   -13,   -29,   -54,   -78, 
// A8                                                H8
};

const array<int,64> PieceSquareTables::knight_eg
{
// A1                                                H1
  -94,   -78,   -62,   -45,   -45,   -62,   -78,   -94, 
  -78,   -62,    -5,    11,    11,    -5,   -62,   -78, 
  -54,    11,    68,    60,    60,    68,    11,   -54, 
  -29,    35,    76,    68,    68,    76,    35,   -29, 
  -29,    35,    76,    68,    68,    76,    35,   -29, 
  -37,    27,   100,    92,    92,   100,    27,   -37, 
  -54,   -29,    43,    60,    60,    43,   -29,   -54, 
  -78,   -54,   -29,   -13,   -13,   -29,   -54,   -78, 
// A8                                                H8
};

const array<int,64> PieceSquareTables::bishop_mg
{
// A1                                                H1
   31,   -22,   -68,   -84,   -84,   -68,   -22,    31, 
  -15,    39,    -7,   -38,   -38,    -7,    39,   -15, 
  -53,     0,    69,    39,    39,    69,     0,   -53, 
  -61,   -22,    46,   100,   100,    46,   -22,   -61, 
  -68,   -30,    39,    92,    92,    39,   -30,   -68, 
  -61,     8,    77,    46,    46,    77,     8,   -61, 
  -22,    46,    16,   -15,   -15,    16,    46,   -22, 
   23,   -15,   -45,   -45,   -45,   -45,   -15,    23, 
// A8                                                H8
};

const array<int,64> PieceSquareTables::bishop_eg
{
// A1                                                H1
   31,   -22,   -68,   -84,   -84,   -68,   -22,    31, 
  -15,    39,    -7,   -38,   -38,    -7,    39,   -15, 
  -53,     0,    69,    39,    39,    69,     0,   -53, 
  -61,   -22,    46,   100,   100,    46,   -22,   -61, 
  -68,   -30,    39,    92,    92,    39,   -30,   -68, 
  -61,     8,    77,    46,    46,    77,     8,   -61, 
  -22,    46,    16,   -15,   -15,    16,    46,   -22, 
   23,   -15,   -45,   -45,   -45,   -45,   -15,    23, 
// A8                                                H8
};

const array<int,64> PieceSquareTables::rook_mg
{
// A1                                                H1
 -100,   -66,   -44,     1,     1,   -44,   -66,  -100, 
  -66,   -44,   -22,    23,    23,   -22,   -44,   -66, 
  -44,   -22,   -10,    34,    34,   -10,   -22,   -44, 
    1,    23,    34,    34,    34,    34,    23,     1, 
   34,    57,    68,    68,    68,    68,    57,    34, 
  -10,    12,    23,    68,    68,    23,    12,   -10, 
  -33,   -10,    12,    57,    57,    12,   -10,   -33, 
  -66,   -33,   -10,    34,    34,   -10,   -33,   -66, 
// A8                                                H8
};

const array<int,64> PieceSquareTables::rook_eg
{
// A1                                                H1
 -100,   -66,   -44,     1,     1,   -44,   -66,  -100, 
  -66,   -44,   -22,    23,    23,   -22,   -44,   -66, 
  -44,   -22,   -10,    34,    34,   -10,   -22,   -44, 
    1,    23,    34,    34,    34,    34,    23,     1, 
   34,    57,    68,    68,    68,    68,    57,    34, 
  -10,    12,    23,    68,    68,    23,    12,   -10, 
  -33,   -10,    12,    57,    57,    12,   -10,   -33, 
  -66,   -33,   -10,    34,    34,   -10,   -33,   -66, 
// A8                                                H8
};

const array<int,64> PieceSquareTables::queen_mg
{
// A1                                                H1
  -57,   -61,   -70,   -45,   -45,   -70,   -61,   -57, 
  -57,   -12,   -20,    -3,    -3,   -20,   -12,   -57, 
  -61,   -16,    30,    46,    46,    30,   -16,   -61, 
  -32,     5,    50,    79,    79,    50,     5,   -32, 
  -12,    26,    71,   100,   100,    71,    26,   -12, 
  -41,    13,    59,    75,    75,    59,    13,   -41, 
  -36,    17,    17,    34,    34,    17,    17,   -36, 
  -36,   -32,   -32,     1,     1,   -32,   -32,   -36, 
// A8                                                H8
};

const array<int,64> PieceSquareTables::queen_eg
{
// A1                                                H1
  -57,   -61,   -70,   -45,   -45,   -70,   -61,   -57, 
  -57,   -12,   -20,    -3,    -3,   -20,   -12,   -57, 
  -61,   -16,    30,    46,    46,    30,   -16,   -61, 
  -32,     5,    50,    79,    79,    50,     5,   -32, 
  -12,    26,    71,   100,   100,    71,    26,   -12, 
  -41,    13,    59,    75,    75,    59,    13,   -41, 
  -36,    17,    17,    34,    34,    17,    17,   -36, 
  -36,   -32,   -32,     1,     1,   -32,   -32,   -36, 
// A8                                                H8
};

const array<int,64> PieceSquareTables::king_mg
{
// A1                                                H1
  100,   100,   100,   100,   100,   100,   100,   100, 
   71,    71,    71,    71,    71,    71,    71,    71, 
   43,    43,    43,    43,    43,    43,    43,    43, 
   14,    14,    14,    14,    14,    14,    14,    14, 
  -14,   -14,   -14,   -14,   -14,   -14,   -14,   -14, 
  -43,   -43,   -43,   -43,   -43,   -43,   -43,   -43, 
  -71,   -71,   -71,   -71,   -71,   -71,   -71,   -71, 
 -100,  -100,  -100,  -100,  -100,  -100,  -100,  -100, 
// A8                                                H8
};

const array<int,64> PieceSquareTables::king_eg
{
// A1                                                H1
  -63,   -50,   -46,   -46,   -46,   -46,   -50,   -63, 
  -50,   -28,   -15,   -10,   -10,   -15,   -28,   -50, 
  -46,   -15,    21,    47,    47,    21,   -15,   -46, 
  -37,     3,    60,    91,    91,    60,     3,   -37, 
  -32,    12,    69,   100,   100,    69,    12,   -32, 
  -24,    21,    56,    82,    82,    56,    21,   -24, 
  -28,     7,    21,    25,    25,    21,     7,   -28, 
  -50,   -28,   -24,   -24,   -24,   -24,   -28,   -50, 
// A8                                                H8
};


PieceSquareTables::PieceSquareTables()
{
  for(auto& s : m_score_mg){s=0;};
  for(auto& s : m_score_eg){s=0;};
}

void PieceSquareTables::init(const ChessPosition& cp)
{
  pcp=&cp;

  for(int i=0;i<64;i++){
    pst_mg[no_piece_id][i]=0;
    pst_mg[white_pawn_id][i]=pawn_mg[i];
    pst_mg[white_knight_id][i]=knight_mg[i];
    pst_mg[white_bishop_id][i]=bishop_mg[i];
    pst_mg[white_rook_id][i]=rook_mg[i];
    pst_mg[white_queen_id][i]=queen_mg[i];
    pst_mg[white_king_id][i]=king_mg[i];
    pst_mg[black_pawn_id][i]=-pawn_mg[i^56];
    pst_mg[black_knight_id][i]=-knight_mg[i^56];
    pst_mg[black_bishop_id][i]=-bishop_mg[i^56];
    pst_mg[black_rook_id][i]=-rook_mg[i^56];
    pst_mg[black_queen_id][i]=-queen_mg[i^56];
    pst_mg[black_king_id][i]=-king_mg[i^56];
  }

  for(int i=0;i<64;i++){
    pst_eg[no_piece_id][i]=0;
    pst_eg[white_pawn_id][i]=pawn_eg[i];
    pst_eg[white_knight_id][i]=knight_eg[i];
    pst_eg[white_bishop_id][i]=bishop_eg[i];
    pst_eg[white_rook_id][i]=rook_eg[i];
    pst_eg[white_queen_id][i]=queen_eg[i];
    pst_eg[white_king_id][i]=king_eg[i];
    pst_eg[black_pawn_id][i]=-pawn_eg[i^56];
    pst_eg[black_knight_id][i]=-knight_eg[i^56];
    pst_eg[black_bishop_id][i]=-bishop_eg[i^56];
    pst_eg[black_rook_id][i]=-rook_eg[i^56];
    pst_eg[black_queen_id][i]=-queen_eg[i^56];
    pst_eg[black_king_id][i]=-king_eg[i^56];
  }


  for(auto& s : m_score_mg){s=0;};
  for(auto& s : m_score_eg){s=0;};
  for(Square s=a1;s<=h8;s++)
    {
      m_score_mg[cp.get_piece_id_from(s)]+=pst_mg[cp.get_piece_id_from(s)][s.id()];
      m_score_eg[cp.get_piece_id_from(s)]+=pst_eg[cp.get_piece_id_from(s)][s.id()];
    }
}

void PieceSquareTables::make_move(const Move& m)
{
  int to{m.to().id()},from{m.from().id()};
  switch(m.code())
    {
    case nullmove:return;
    case normal_move:
      m_score_mg[m.piece_id()]+=(pst_mg[m.piece_id()][to]-pst_mg[m.piece_id()][from]);
      m_score_eg[m.piece_id()]+=(pst_eg[m.piece_id()][to]-pst_eg[m.piece_id()][from]);
      m_score_mg[m.capture_id()]-=pst_mg[m.capture_id()][to];
      m_score_eg[m.capture_id()]-=pst_eg[m.capture_id()][to];
      return;
    case en_passant:
      m_score_mg[m.piece_id()]+=(pst_mg[m.piece_id()][to]-pst_mg[m.piece_id()][from]);
      m_score_eg[m.piece_id()]+=(pst_eg[m.piece_id()][to]-pst_eg[m.piece_id()][from]);
      if(to>from){
	m_score_mg[m.capture_id()]-=pst_mg[m.capture_id()][to-8];
	m_score_eg[m.capture_id()]-=pst_eg[m.capture_id()][to-8];
      }else{
	m_score_mg[m.capture_id()]-=pst_mg[m.capture_id()][to+8];
	m_score_eg[m.capture_id()]-=pst_eg[m.capture_id()][to+8];
      }
      return;
    case castling:
      if(from<8){
	m_score_mg[white_king_id]-=pst_mg[white_king_id][from];
	m_score_eg[white_king_id]-=pst_eg[white_king_id][from];
	m_score_mg[white_rook_id]-=pst_mg[white_rook_id][to];
	m_score_eg[white_rook_id]-=pst_eg[white_rook_id][to];
	if(to>from){
	  m_score_mg[white_king_id]+=pst_mg[white_king_id][g1.id()];
	  m_score_eg[white_king_id]+=pst_eg[white_king_id][g1.id()];
	  m_score_mg[white_rook_id]+=pst_mg[white_rook_id][f1.id()];
	  m_score_eg[white_rook_id]+=pst_eg[white_rook_id][f1.id()];
	}else{
	  m_score_mg[white_king_id]+=pst_mg[white_king_id][c1.id()];
	  m_score_eg[white_king_id]+=pst_eg[white_king_id][c1.id()];
	  m_score_mg[white_rook_id]+=pst_mg[white_rook_id][d1.id()];
	  m_score_eg[white_rook_id]+=pst_eg[white_rook_id][d1.id()];
	}
      }else{
	m_score_mg[black_king_id]-=pst_mg[black_king_id][from];
	m_score_eg[black_king_id]-=pst_eg[black_king_id][from];
	m_score_mg[black_rook_id]-=pst_mg[black_rook_id][to];
	m_score_eg[black_rook_id]-=pst_eg[black_rook_id][to];
	if(to>from){
	  m_score_mg[black_king_id]+=pst_mg[black_king_id][g8.id()];
	  m_score_eg[black_king_id]+=pst_eg[black_king_id][g8.id()];
	  m_score_mg[black_rook_id]+=pst_mg[black_rook_id][f8.id()];
	  m_score_eg[black_rook_id]+=pst_eg[black_rook_id][f8.id()];
	}else{
	  m_score_mg[black_king_id]+=pst_mg[black_king_id][c8.id()];
	  m_score_eg[black_king_id]+=pst_eg[black_king_id][c8.id()];
	  m_score_mg[black_rook_id]+=pst_mg[black_rook_id][d8.id()];
	  m_score_eg[black_rook_id]+=pst_eg[black_rook_id][d8.id()];
	}
      }
      return;
    case promo_queen:
      m_score_mg[m.capture_id()]-=pst_mg[m.capture_id()][to];
      m_score_eg[m.capture_id()]-=pst_eg[m.capture_id()][to];
      m_score_mg[m.piece_id()]-=pst_mg[m.piece_id()][from];
      m_score_eg[m.piece_id()]-=pst_eg[m.piece_id()][from];
      if(to>from){
	m_score_mg[white_queen_id]+=pst_mg[white_queen_id][to];
	m_score_eg[white_queen_id]+=pst_eg[white_queen_id][to];
      }else{
	m_score_mg[black_queen_id]+=pst_mg[black_queen_id][to];
	m_score_eg[black_queen_id]+=pst_eg[black_queen_id][to];
      }
      return;
    case promo_rook:
      m_score_mg[m.capture_id()]-=pst_mg[m.capture_id()][to];
      m_score_eg[m.capture_id()]-=pst_eg[m.capture_id()][to];
      m_score_mg[m.piece_id()]-=pst_mg[m.piece_id()][from];
      m_score_eg[m.piece_id()]-=pst_eg[m.piece_id()][from];
      if(to>from){
	m_score_mg[white_rook_id]+=pst_mg[white_rook_id][to];
	m_score_eg[white_rook_id]+=pst_eg[white_rook_id][to];
      }else{
	m_score_mg[black_rook_id]+=pst_mg[black_rook_id][to];
	m_score_eg[black_rook_id]+=pst_eg[black_rook_id][to];
      }
      return;
    case promo_bishop:
      m_score_mg[m.capture_id()]-=pst_mg[m.capture_id()][to];
      m_score_eg[m.capture_id()]-=pst_eg[m.capture_id()][to];
      m_score_mg[m.piece_id()]-=pst_mg[m.piece_id()][from];
      m_score_eg[m.piece_id()]-=pst_eg[m.piece_id()][from];
      if(to>from){
	m_score_mg[white_bishop_id]+=pst_mg[white_bishop_id][to];
	m_score_eg[white_bishop_id]+=pst_eg[white_bishop_id][to];
      }else{
	m_score_mg[black_bishop_id]+=pst_mg[black_bishop_id][to];
	m_score_eg[black_bishop_id]+=pst_eg[black_bishop_id][to];
      }
      return;
    case promo_knight:
      m_score_mg[m.capture_id()]-=pst_mg[m.capture_id()][to];
      m_score_eg[m.capture_id()]-=pst_eg[m.capture_id()][to];
      m_score_mg[m.piece_id()]-=pst_mg[m.piece_id()][from];
      m_score_eg[m.piece_id()]-=pst_eg[m.piece_id()][from];
      if(to>from){
	m_score_mg[white_knight_id]+=pst_mg[white_knight_id][to];
	m_score_eg[white_knight_id]+=pst_eg[white_knight_id][to];
      }else{
	m_score_mg[black_knight_id]+=pst_mg[black_knight_id][to];
	m_score_eg[black_knight_id]+=pst_eg[black_knight_id][to];
      }
      return;
    }
}

void PieceSquareTables::unmake_move(const Move& m)
{
  int to{m.to().id()},from{m.from().id()};
  switch(m.code())
    {
    case nullmove:return;
    case normal_move:
      m_score_mg[m.piece_id()]-=(pst_mg[m.piece_id()][to]-pst_mg[m.piece_id()][from]);
      m_score_eg[m.piece_id()]-=(pst_eg[m.piece_id()][to]-pst_eg[m.piece_id()][from]);
      m_score_mg[m.capture_id()]+=pst_mg[m.capture_id()][to];
      m_score_eg[m.capture_id()]+=pst_eg[m.capture_id()][to];
      return;
    case en_passant:
      m_score_mg[m.piece_id()]-=(pst_mg[m.piece_id()][to]-pst_mg[m.piece_id()][from]);
      m_score_eg[m.piece_id()]-=(pst_eg[m.piece_id()][to]-pst_eg[m.piece_id()][from]);
      if(to>from){
	m_score_mg[m.capture_id()]+=pst_mg[m.capture_id()][to-8];
	m_score_eg[m.capture_id()]+=pst_eg[m.capture_id()][to-8];
      }else{
	m_score_mg[m.capture_id()]+=pst_mg[m.capture_id()][to+8];
	m_score_eg[m.capture_id()]+=pst_eg[m.capture_id()][to+8];
      }
      return;
    case castling:
      if(from<8){
	m_score_mg[white_king_id]+=pst_mg[white_king_id][from];
	m_score_eg[white_king_id]+=pst_eg[white_king_id][from];
	m_score_mg[white_rook_id]+=pst_mg[white_rook_id][to];
	m_score_eg[white_rook_id]+=pst_eg[white_rook_id][to];
	if(to>from){
	  m_score_mg[white_king_id]-=pst_mg[white_king_id][g1.id()];
	  m_score_eg[white_king_id]-=pst_eg[white_king_id][g1.id()];
	  m_score_mg[white_rook_id]-=pst_mg[white_rook_id][f1.id()];
	  m_score_eg[white_rook_id]-=pst_eg[white_rook_id][f1.id()];
	}else{
	  m_score_mg[white_king_id]-=pst_mg[white_king_id][c1.id()];
	  m_score_eg[white_king_id]-=pst_eg[white_king_id][c1.id()];
	  m_score_mg[white_rook_id]-=pst_mg[white_rook_id][d1.id()];
	  m_score_eg[white_rook_id]-=pst_eg[white_rook_id][d1.id()];
	}
      }else{
	m_score_mg[black_king_id]+=pst_mg[black_king_id][from];
	m_score_eg[black_king_id]+=pst_eg[black_king_id][from];
	m_score_mg[black_rook_id]+=pst_mg[black_rook_id][to];
	m_score_eg[black_rook_id]+=pst_eg[black_rook_id][to];
	if(to>from){
	  m_score_mg[black_king_id]-=pst_mg[black_king_id][g8.id()];
	  m_score_eg[black_king_id]-=pst_eg[black_king_id][g8.id()];
	  m_score_mg[black_rook_id]-=pst_mg[black_rook_id][f8.id()];
	  m_score_eg[black_rook_id]-=pst_eg[black_rook_id][f8.id()];
	}else{
	  m_score_mg[black_king_id]-=pst_mg[black_king_id][c8.id()];
	  m_score_eg[black_king_id]-=pst_eg[black_king_id][c8.id()];
	  m_score_mg[black_rook_id]-=pst_mg[black_rook_id][d8.id()];
	  m_score_eg[black_rook_id]-=pst_eg[black_rook_id][d8.id()];
	}
      }
      return;
    case promo_queen:
      m_score_mg[m.capture_id()]+=pst_mg[m.capture_id()][to];
      m_score_eg[m.capture_id()]+=pst_eg[m.capture_id()][to];
      m_score_mg[m.piece_id()]+=pst_mg[m.piece_id()][from];
      m_score_eg[m.piece_id()]+=pst_eg[m.piece_id()][from];
      if(to>from){
	m_score_mg[white_queen_id]-=pst_mg[white_queen_id][to];
	m_score_eg[white_queen_id]-=pst_eg[white_queen_id][to];
      }else{
	m_score_mg[black_queen_id]-=pst_mg[black_queen_id][to];
	m_score_eg[black_queen_id]-=pst_eg[black_queen_id][to];
      }
      return;
    case promo_rook:
      m_score_mg[m.capture_id()]+=pst_mg[m.capture_id()][to];
      m_score_eg[m.capture_id()]+=pst_eg[m.capture_id()][to];
      m_score_mg[m.piece_id()]+=pst_mg[m.piece_id()][from];
      m_score_eg[m.piece_id()]+=pst_eg[m.piece_id()][from];
      if(to>from){
	m_score_mg[white_rook_id]-=pst_mg[white_rook_id][to];
	m_score_eg[white_rook_id]-=pst_eg[white_rook_id][to];
      }else{
	m_score_mg[black_rook_id]-=pst_mg[black_rook_id][to];
	m_score_eg[black_rook_id]-=pst_eg[black_rook_id][to];
      }
      return;
    case promo_bishop:
      m_score_mg[m.capture_id()]+=pst_mg[m.capture_id()][to];
      m_score_eg[m.capture_id()]+=pst_eg[m.capture_id()][to];
      m_score_mg[m.piece_id()]+=pst_mg[m.piece_id()][from];
      m_score_eg[m.piece_id()]+=pst_eg[m.piece_id()][from];
      if(to>from){
	m_score_mg[white_bishop_id]-=pst_mg[white_bishop_id][to];
	m_score_eg[white_bishop_id]-=pst_eg[white_bishop_id][to];
      }else{
	m_score_mg[black_bishop_id]-=pst_mg[black_bishop_id][to];
	m_score_eg[black_bishop_id]-=pst_eg[black_bishop_id][to];
      }
      return;
    case promo_knight:
      m_score_mg[m.capture_id()]+=pst_mg[m.capture_id()][to];
      m_score_eg[m.capture_id()]+=pst_eg[m.capture_id()][to];
      m_score_mg[m.piece_id()]+=pst_mg[m.piece_id()][from];
      m_score_eg[m.piece_id()]+=pst_eg[m.piece_id()][from];
      if(to>from){
	m_score_mg[white_knight_id]-=pst_mg[white_knight_id][to];
	m_score_eg[white_knight_id]-=pst_eg[white_knight_id][to];
      }else{
	m_score_mg[black_knight_id]-=pst_mg[black_knight_id][to];
	m_score_eg[black_knight_id]-=pst_eg[black_knight_id][to];
      }
      return;
    }
}

int PieceSquareTables::get_pawn_mg()const
{
  return m_score_mg[white_pawn_id]+m_score_mg[black_pawn_id];
}

int PieceSquareTables::get_pawn_eg()const
{
  return m_score_eg[white_pawn_id]+m_score_eg[black_pawn_id];
}

int PieceSquareTables::get_knight_mg()const
{
  return m_score_mg[white_knight_id]+m_score_mg[black_knight_id];
}

int PieceSquareTables::get_knight_eg()const
{
  return m_score_eg[white_knight_id]+m_score_eg[black_knight_id];
}

int PieceSquareTables::get_bishop_mg()const
{
  return m_score_mg[white_bishop_id]+m_score_mg[black_bishop_id];
}

int PieceSquareTables::get_bishop_eg()const
{
  return m_score_eg[white_bishop_id]+m_score_eg[black_bishop_id];
}

int PieceSquareTables::get_rook_mg()const
{
  return m_score_mg[white_rook_id]+m_score_mg[black_rook_id];
}

int PieceSquareTables::get_rook_eg()const
{
  return m_score_eg[white_rook_id]+m_score_eg[black_rook_id];
}

int PieceSquareTables::get_queen_mg()const
{
  return m_score_mg[white_queen_id]+m_score_mg[black_queen_id];
}

int PieceSquareTables::get_queen_eg()const
{
  return m_score_eg[white_queen_id]+m_score_eg[black_queen_id];
}

int PieceSquareTables::get_king_mg()const
{
  return m_score_mg[white_king_id]+m_score_mg[black_king_id];
}

int PieceSquareTables::get_king_eg()const
{
  return m_score_eg[white_king_id]+m_score_eg[black_king_id];
}
