/*

Copyright (C) 2018 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include<iostream>
#include<iomanip>
#include<new>
#include<sstream>
#include<stdexcept>
#include"transposition.h"
using std::bad_alloc;
using std::logic_error;
using std::cout;
using std::endl;
using std::hex;
using std::dec;
using std::ostringstream;



TranspositionTable::TranspositionTable()
{
  m_number_of_buckets=1<<21;             // default 128 Mb (= 2M * (4 * 16 bytes))
  m_bucket=new Bucket[m_number_of_buckets]; 
  clear();
}

TranspositionTable::~TranspositionTable()
{
  delete[] m_bucket;
}

unsigned char* TranspositionTable::get_uchar_adress()const
{
  return reinterpret_cast<unsigned char*>(m_bucket);
}

int TranspositionTable::get_size()
{
  return (m_number_of_buckets*sizeof(Bucket))>>20;    // returns size in Mb
}

void TranspositionTable::set_size(int size)
{
  long long x=size;
  if(x<6){x=6;}
  x<<=20;                // conversion Mb to bytes
  x/=sizeof(Bucket);     // number of buckets (1 bucket == 4 * 16 bytes)
  uint64_t nob=1;
  while(x>1){nob<<=1;x>>=1;}
  if(nob==m_number_of_buckets){return;}else{m_number_of_buckets=nob;}
  delete[] m_bucket;
  m_bucket=new Bucket[m_number_of_buckets]; 
  clear();  
}

void TranspositionTable::set_threshold(int t)
{
  m_threshold=t>>6;
}

int TranspositionTable::get_threshold()const
{
  return 64*m_threshold;
}

int TranspositionTable::hashfull(unsigned char x)const
{
  int hashfull{0};
  for(unsigned i=0;i < 1000 && i<m_number_of_buckets;i++){
    if(m_bucket[i].has_date(x)){hashfull++;}
  }
  return hashfull;
}

void TranspositionTable::clear()
{
  for(unsigned i=m_threshold+1;i<m_number_of_buckets;i++){
    m_bucket[i].clear();
  }
}

bool TranspositionTable::read(TTrecord& x)
{
  if(m_enabled){
    uint64_t key=x.get_key();
    uint64_t k=key & (m_number_of_buckets-1);
    if(k>m_threshold){
      return m_bucket[k].read(x);
    }
  }
  return false;
}

void TranspositionTable::write(const TTrecord& x)
{
  if(m_enabled){
    uint64_t key=x.get_key();
    uint64_t k=key & (m_number_of_buckets-1);
    if(k>m_threshold){
      m_bucket[k].write(x);
    }
  }
}

bool TranspositionTable::enabled()const
{
  return m_enabled;
}

void TranspositionTable::set_enabled(bool x)
{
  m_enabled=x;
}

