/*

Copyright (C) 2018 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include<iostream>
#include"ttrecord.h"
using std::endl;
using std::hex;
using std::dec;


TTrecord::TTrecord()
{
  key=0;
  bound=0;
  sd=0;
  score=0;
  date=0;
  move.clear();
}

void TTrecord::clear()
{
  key=0;
  bound=0;
  sd=0;
  score=0;
  date=0;
  move.clear();
}

bool TTrecord::operator==(const TTrecord& x)
{
  return key==x.key && move==x.move && score==x.score && bound==x.bound && sd==x.sd && date==x.date;
}

bool TTrecord::operator!=(const TTrecord& x)
{
  return key!=x.key || move!=x.move || score!=x.score || bound!=x.bound || sd!=x.sd || date!=x.date;
}

const uint64_t& TTrecord::get_key()const
{
  return key;
}

void TTrecord::set_key(const uint64_t& x)
{
  key=x;
}

const TTmove& TTrecord::get_move()const
{
  return move;
}

void TTrecord::set_move(const TTmove& x)
{
  move=x;
}

int TTrecord::get_score(const int& ply)const
{
  if(score>=mate_value-max_search_depth){
    return score-ply;
  }else if(score<=-mate_value+max_search_depth){
    return score+ply;
  }else{
    return score;
  }
}

void TTrecord::set_score(const int& x,const int& ply)
{
  score=static_cast<short int>(x);
  if(score>=mate_value-max_search_depth){
    score+=ply;
  }else if(score<=-mate_value+max_search_depth){
    score-=ply;
  }
}

bool TTrecord::is_exact_value()const
{
  return bound==exact_value;
}

bool TTrecord::is_upperbound()const
{
  return bound==upperbound;
}

bool TTrecord::is_lowerbound()const
{
  return bound==lowerbound;
}

void TTrecord::set_bound(bound_t x)
{
  bound=x;
}


const unsigned char& TTrecord::get_draft()const
{
  return sd;
}

void TTrecord::set_draft(const unsigned char& x)
{
  sd=x;
}

void TTrecord::set_date(const unsigned char& x)
{
  date=x;
}

const unsigned char& TTrecord::get_date()const
{
  return date;
}
