/*

Copyright (C) 2018 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#ifndef MLIST_H
#define MLIST_H

#include<utility>
#include<iterator>
#include"move.h"
#include"../search/history.h"
using std::pair;

class ChessPosition;
class Analysis;



class MList
{
 private:
  ChessPosition* pcp{nullptr};
  Analysis* pana{nullptr};

  vector<Move> m_hashtable_move;
  vector<Move> m_good_capture;
  vector<Move> m_killer;
  vector<Move> m_killer_2;
  vector<Move> m_counter_move;
  vector<Move> m_historical_move;
  vector<Move> m_fertile_move_1;
  vector<Move> m_fertile_move_2;
  vector<Move> m_fertile_move_3;
  vector<Move> m_to_centre;
  vector<Move> m_to_bigcentre;
  vector<Move> m_losing_capture;
  vector<Move> m_xnormal_move;

  size_t m_size{0};
  Move *m_vm{nullptr};
  bool m_allocate_mem{false};
  unsigned m_current{0};
  Move m_htm;                     // hashtable move
  Move m_ka,m_kb,m_ka_2,m_kb_2;   // killers
  bool m_log{false};
  
 public:
  MList(bool=false);
  ~MList();
  void set_vm(Move* pvm);
  Move* get_vm();
  Move& operator[](int);
  void set_chessposition(ChessPosition&);
  void set_analysis(Analysis*);
  void set_searchmoves(const vector<Move>&);
  void set_hashtable_move(const Move&);
  void set_killers(const Move&,const Move&);
  void set_killers_2(const Move&,const Move&);
  void set_log(bool);
  void set_xcodes();
  void sort();
  void add(const Move&);
  void remove(const Move&);
  void clear();
  const Move& first();
  const Move& next();
  Move& back();
  void insert(vector<Move>::iterator,vector<Move>::iterator);
  void pop_back();
  void push_back(const Move&);
  unsigned int size()const;
  Move* begin(){return m_vm;};
  Move* end(){return m_vm+m_size;};
  string dump()const;
};



#endif
