/*

Copyright (C) 2018 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include<stdexcept>
#include<iostream>
#include<utility>
#include"../common/numerals.h"
#include"movegenerator.h"
#include"../common/prebb.h"
using std::cout;
using std::endl;
using std::move;
using std::logic_error;


MoveGenerator::MoveGenerator(ChessPosition& x): cp(x), m_con(x.connections()), m_atta(x.attacks())
{
}

bool MoveGenerator::is_legal_pawnmove(const Move& m)const
{
  BitBoard bb_first{m.from().bitboard()},bb_second{m.to().bitboard()},ka;
  if(cp.check()){
    for(auto s : m_king_attackers){
      if(m_con.line_segment(m_ownkingsfield.get_unique_square(),s) & m_pieces & ~bb_first){continue;}
      if(m_con.closed_line_segment(m_ownkingsfield.get_unique_square(),s) & bb_second){continue;}
      if(m.code()==en_passant){
	Square captured_pawn_square;
	if(m.piece_id()==white_pawn_id){
	  bb_first.set_square(m.to()-8);    // bb_first contains two squares now
	  captured_pawn_square=m.to()-8;
	}
	else{
	  bb_first.set_square(m.to()+8);    // bb_first contains two squares now
	  captured_pawn_square=m.to()+8;
	}
	for(auto s : m_king_attackers){
	  if(m_con.line_segment(m_ownkingsfield.get_unique_square(),s) & m_pieces & ~bb_first){continue;}
	  if(m_con.closed_line_segment(m_ownkingsfield.get_unique_square(),s) & bb_second){continue;}
	  if(captured_pawn_square==s){continue;}
	  return false;
	}
	continue;
      }
      return false;
    }
  }else if(m.code()==en_passant){
    Square captured_pawn_square;
    if(m.piece_id()==white_pawn_id){
      bb_first.set_square(m.to()-8);    // bb_first contains two squares now
      captured_pawn_square=m.to()-8;
    }
    else{
      bb_first.set_square(m.to()+8);    // bb_first contains two squares now
      captured_pawn_square=m.to()+8;
    }
    for(auto s : m_king_attackers){
      if(m_con.line_segment(m_ownkingsfield.get_unique_square(),s) & m_pieces & ~bb_first){continue;}
      if(m_con.closed_line_segment(m_ownkingsfield.get_unique_square(),s) & bb_second){continue;}
      if(captured_pawn_square==s){continue;}
      return false;
    }
  }else if(m.from().bitboard()&m_pinned_pieces){
    for(auto s : m_king_attackers){
      if(m_con.line_segment(m_ownkingsfield.get_unique_square(),s) & bb_first)
	{
	  if(m_con.line_segment(m_ownkingsfield.get_unique_square(),s) & m_pieces & ~bb_first){continue;}
	  if(m_con.closed_line_segment(m_ownkingsfield.get_unique_square(),s) & bb_second){continue;}
	  return false;
	}
    }
  }
  return true;
}

template<int COLOR>
bool MoveGenerator::is_legal_kingmove(const Move& m)const
{
  BitBoard bb_first{m.from().bitboard()},bb_second{m.to().bitboard()};
  BitBoard ka=COLOR==white?cp.black_attackers(m.to().bitboard()):cp.white_attackers(m.to().bitboard());
  BitBoard kba=COLOR==white?cp.black_blocked_attackers(m.to().bitboard()):cp.white_blocked_attackers(m.to().bitboard());
  if(cp.check()){
    while(kba){
      auto s=kba.get_reset_square().get_unique_square();
      if(m_con.line_segment(m.to(),s) & m_pieces & ~bb_first){continue;}
      return false;
    }
  }else if(ka){
    return false;
  }
  return true;
}

bool MoveGenerator::is_legal(const Move& m)const
{
  BitBoard bb_first{m.from().bitboard()},bb_second{m.to().bitboard()};
  if(cp.check()){
    for(auto s : m_king_attackers){
      if(m_con.line_segment(m_ownkingsfield.get_unique_square(),s) & m_pieces & ~bb_first){continue;}
      if(m_con.closed_line_segment(m_ownkingsfield.get_unique_square(),s) & bb_second){continue;}
      return false;
    }
  }else if(m.from().bitboard()&m_pinned_pieces){
    for(auto s : m_king_attackers){
      if(m_con.line_segment(m_ownkingsfield.get_unique_square(),s) & bb_first){
	if(m_con.line_segment(m_ownkingsfield.get_unique_square(),s) & m_pieces & ~bb_first){continue;}
	if(m_con.closed_line_segment(m_ownkingsfield.get_unique_square(),s) & bb_second){continue;}
	return false;
      }
    }
  }
  return true;
}

template<int COLOR>
bool MoveGenerator::discovered_check(const BitBoard& bishopattacks,const BitBoard& rookattacks)
{
  bool r{false};
  r=COLOR==white
    ?
    ((cp.white_queen() | cp.white_rook()) & rookattacks)||((cp.white_queen() | cp.white_bishop()) & bishopattacks)
    :
    ((cp.black_queen() | cp.black_rook()) & rookattacks)||((cp.black_queen() | cp.black_bishop()) & bishopattacks)
    ;
  return r;
}

template<int COLOR>
void MoveGenerator::set_checkstatus(Move& m)
{
  m.check(false);
  BitBoard bb_from=m.from().bitboard();
  BitBoard bb_to=m.to().bitboard();

  m_pawnattacks=COLOR==white
    ?(cp.black_king()>>7 & not_a_file) | (cp.black_king()>>9 & not_h_file)
    :(cp.white_king()<<7 & not_h_file) | (cp.white_king()<<9 & not_a_file);
  
  m.xcode(xnormal_move);
  switch(m.code()){
  case normal_move:
    if((bb_from & m_discovering_pieces)){
      if(!m.piece().pawn() && !m.piece().king()){m.check(true);}
      else{
	BitBoard rookattacks=m_atta.rook(m_opponentkingsfield,(m_pieces & ~bb_from) | bb_to);
	BitBoard bishopattacks=m_atta.bishop(m_opponentkingsfield,(m_pieces & ~bb_from) | bb_to);
	BitBoard queen_and_rook=COLOR==white
	  ?
	  (cp.white_queen() | cp.white_rook()) & rookattacks
	  :
	  (cp.black_queen() | cp.black_rook()) & rookattacks
	  ;
	BitBoard queen_and_bishop=COLOR==white
	  ?
	  (cp.white_queen() | cp.white_bishop()) & bishopattacks
	  :
	  (cp.black_queen() | cp.black_bishop()) & bishopattacks
	  ;
	if(queen_and_rook | queen_and_bishop){m.check(true);}
      }
    }
    if((m.piece().queen() || m.piece().rook()) && (bb_to & m_rookattacks)){m.check(true);}
    else if((m.piece().queen() || m.piece().bishop()) && (m.to().bitboard() & m_bishopattacks)){m.check(true);}
    else if(m.piece().knight() && (m.to().bitboard() & m_knightattacks)){m.check(true);}
    else if(m.piece().pawn() && (m.to().bitboard() & m_pawnattacks)){m.check(true);}
    break;
  case promo_queen:
    {
      BitBoard rookattacks=m_atta.rook(m_opponentkingsfield,(m_pieces & ~bb_from) | bb_to);
      BitBoard bishopattacks=m_atta.bishop(m_opponentkingsfield,(m_pieces & ~bb_from) | bb_to);
      if(bb_to & rookattacks){m.check(true);}
      else if(bb_to & bishopattacks){m.check(true);}
      if(discovered_check<COLOR>(bishopattacks,rookattacks)){m.check(true);}
    }
    break;
  case promo_rook:
    {
      BitBoard rookattacks=m_atta.rook(m_opponentkingsfield,(m_pieces & ~bb_from) | bb_to);
      BitBoard bishopattacks=m_atta.bishop(m_opponentkingsfield,(m_pieces & ~bb_from) | bb_to);
      if(bb_to & rookattacks){m.check(true);}
      if(discovered_check<COLOR>(bishopattacks,rookattacks)){m.check(true);}
    }
    break;
  case promo_bishop:
    {
      BitBoard rookattacks=m_atta.rook(m_opponentkingsfield,(m_pieces & ~bb_from) | bb_to);
      BitBoard bishopattacks=m_atta.bishop(m_opponentkingsfield,(m_pieces & ~bb_from) | bb_to);
      if(bb_to & bishopattacks){m.check(true);}
      if(discovered_check<COLOR>(bishopattacks,rookattacks)){m.check(true);}
    }
    break;
  case promo_knight:
    {
      BitBoard rookattacks=m_atta.rook(m_opponentkingsfield,(m_pieces & ~bb_from) | bb_to);
      BitBoard bishopattacks=m_atta.bishop(m_opponentkingsfield,(m_pieces & ~bb_from) | bb_to);
      if(bb_to & m_knightattacks){m.check(true);}
      if(discovered_check<COLOR>(bishopattacks,rookattacks)){m.check(true);}
    }
    break;
  case castling:
    {
      BitBoard rookattacks=m_atta.rook(m_opponentkingsfield,m_pieces & ~bb_from & ~bb_to);
      if(m.to()>m.from() && m.from()<h1 && (f1.bitboard() & rookattacks)){m.check(true);}
      else if(m.to()<m.from() && m.from()<h1 && (d1.bitboard() & rookattacks)){m.check(true);}
      else if(m.to()>m.from() && m.from()>a8 && (f8.bitboard() & rookattacks)){m.check(true);}
      else if(m.to()<m.from() && m.from()>a8 && (d8.bitboard() & rookattacks)){m.check(true);}
    }
    break;
  case en_passant:
    if(m.piece().white_piece()){
      BitBoard ra=m_atta.rook(m_opponentkingsfield,(m_pieces & ~m.from().bitboard() & ~(m.to()-8).bitboard()) | m.to().bitboard());
      if((ra & cp.white_queen())||(ra & cp.white_rook())){m.check(true);}
      BitBoard ba=m_atta.bishop(m_opponentkingsfield,(m_pieces & ~m.from().bitboard() & ~(m.to()-8).bitboard()) | m.to().bitboard());
      if((ba & cp.white_queen())||(ba & cp.white_bishop())){m.check(true);}
    }else{
      BitBoard ra=m_atta.rook(m_opponentkingsfield,(m_pieces & ~m.from().bitboard() & ~(m.to()+8).bitboard()) | m.to().bitboard());
      if((ra & cp.black_queen())||(ra & cp.black_rook())){m.check(true);}
      BitBoard ba=m_atta.bishop(m_opponentkingsfield,(m_pieces & ~m.from().bitboard() & ~(m.to()+8).bitboard()) | m.to().bitboard());
      if((ba & cp.black_queen())||(ba & cp.black_bishop())){m.check(true);}
    }
    if(m.piece().pawn() && (m.to().bitboard() & m_pawnattacks)){m.check(true);}
    break;
  case nullmove:break;
  }
}

template<class T>
void MoveGenerator::generate_moves(T& mlist)
{
  m_movelist.clear();
  mlist.clear();
  mlist.set_chessposition(cp);

  //mmm
  m_pinned_pieces=cp.pinned_pieces();
  m_discovering_pieces=cp.discovering_pieces();
  m_pieces=cp.pieces();
  //mmm

  if(cp.white_to_move()){
    m_own_pieces=cp.white_pieces();
    m_opponent_pieces=cp.black_pieces();
    m_ownkingsfield=cp.white_king();
    m_opponentkingsfield=cp.black_king();
    m_king_attackers=cp.black_blocked_attackers(m_ownkingsfield).get_squares();
    m_rookattacks=m_atta.rook(m_opponentkingsfield,m_pieces);
    m_bishopattacks=m_atta.bishop(m_opponentkingsfield,m_pieces);
    m_knightattacks=m_atta.knight(m_opponentkingsfield);  
    generate_movetable<white>();
    generate_captures<white>(m_movelist);
    generate_quiet_moves<white>(m_movelist);
    mlist.insert(m_movelist.begin(),m_movelist.end());
  }else{
    m_own_pieces=cp.black_pieces();
    m_opponent_pieces=cp.white_pieces();
    m_ownkingsfield=cp.black_king();
    m_opponentkingsfield=cp.white_king();
    m_king_attackers=cp.white_blocked_attackers(m_ownkingsfield).get_squares();
    m_rookattacks=m_atta.rook(m_opponentkingsfield,m_pieces);
    m_bishopattacks=m_atta.bishop(m_opponentkingsfield,m_pieces);
    m_knightattacks=m_atta.knight(m_opponentkingsfield);  
    generate_movetable<black>();
    generate_captures<black>(m_movelist);
    generate_quiet_moves<black>(m_movelist);
    mlist.insert(m_movelist.begin(),m_movelist.end());
  }
}

//----------------------------------- attacks --------------------------------------------
//
template<class T>
void MoveGenerator::generate_quiescence_moves(T& mlist)
{
  m_movelist.clear();
  mlist.clear();
  mlist.set_chessposition(cp);

  //mmm
  m_pinned_pieces=cp.pinned_pieces();
  m_discovering_pieces=cp.discovering_pieces();
  m_pieces=cp.pieces();
  //mmm
  
  if(cp.white_to_move()){
    m_own_pieces=cp.white_pieces();
    m_opponent_pieces=cp.black_pieces();
    m_ownkingsfield=cp.white_king();
    m_opponentkingsfield=cp.black_king();
    m_king_attackers=cp.black_blocked_attackers(m_ownkingsfield).get_squares();
    m_rookattacks=m_atta.rook(m_opponentkingsfield,m_pieces);
    m_bishopattacks=m_atta.bishop(m_opponentkingsfield,m_pieces);
    m_knightattacks=m_atta.knight(m_opponentkingsfield);  
    generate_movetable<white>();
    generate_captures<white>(m_movelist);
    mlist.insert(m_movelist.begin(),m_movelist.end());
  }else{
    m_own_pieces=cp.black_pieces();
    m_opponent_pieces=cp.white_pieces();
    m_ownkingsfield=cp.black_king();
    m_opponentkingsfield=cp.white_king();
    m_king_attackers=cp.white_blocked_attackers(m_ownkingsfield).get_squares();
    m_rookattacks=m_atta.rook(m_opponentkingsfield,m_pieces);
    m_bishopattacks=m_atta.bishop(m_opponentkingsfield,m_pieces);
    m_knightattacks=m_atta.knight(m_opponentkingsfield);  
    generate_movetable<black>();
    generate_captures<black>(m_movelist);
    mlist.insert(m_movelist.begin(),m_movelist.end());
  }
}

//-------------------------------- moves table  ( no castling, no EP) -----------------------------
//
template<int COLOR>
void MoveGenerator::generate_movetable()
{
  constexpr int PAWN=COLOR?white_pawn_id:black_pawn_id;
  constexpr int KNIGHT=COLOR?white_knight_id:black_knight_id;
  constexpr int BISHOP=COLOR?white_bishop_id:black_bishop_id;
  constexpr int ROOK=COLOR?white_rook_id:black_rook_id;
  constexpr int QUEEN=COLOR?white_queen_id:black_queen_id;
  constexpr int KING=COLOR?white_king_id:black_king_id;
  
  for(auto& m : m_moves){m.clear();}
  BitBoard bb;
  
  bb=cp.piece(PAWN);
  while(bb){
    Square sq=bb.get_reset_square().get_unique_square();
    BitBoard bbp=sq.bitboard();
    m_moves[sq.id()]=COLOR?
      (bbp<<7 & ~h_file & cp.black_pieces()) |
      (bbp<<8 & ~cp.pieces()) |
      ((bbp<<8 & ~cp.pieces())&rank3)<<8 & ~cp.pieces() |
      (bbp<<9 & ~a_file & cp.black_pieces())
      :
      (bbp>>7 & ~a_file & cp.white_pieces()) |
      (bbp>>8 & ~cp.pieces()) |
      ((bbp>>8 & ~cp.pieces())&rank6)>>8 & ~cp.pieces() |
      (bbp>>9 & ~h_file & cp.white_pieces());
  }
  bb=cp.piece(KNIGHT);
  while(bb){
    Square sq=bb.get_reset_square().get_unique_square();
    m_moves[sq.id()]=m_atta.knight(sq);
  }
  bb=cp.piece(BISHOP);
  while(bb){
    Square sq=bb.get_reset_square().get_unique_square();
    m_moves[sq.id()]=m_atta.bishop(sq,m_pieces);
  }
  bb=cp.piece(ROOK);
  while(bb){
    Square sq=bb.get_reset_square().get_unique_square();
    m_moves[sq.id()]=m_atta.rook(sq,m_pieces);
  }
  bb=cp.piece(QUEEN);
  while(bb){
    Square sq=bb.get_reset_square().get_unique_square();
    m_moves[sq.id()]=m_atta.queen(sq,m_pieces);
  }
  bb=cp.piece(KING);
  while(bb){
    Square sq=bb.get_reset_square().get_unique_square();
    m_moves[sq.id()]=m_atta.king(sq);
  }  
}

template<int COLOR>
void MoveGenerator::generate_captures(vector<Move>& r)
{
  Move m;
  constexpr int PAWN=COLOR?white_pawn_id:black_pawn_id;
  constexpr int KNIGHT=COLOR?white_knight_id:black_knight_id;
  constexpr int BISHOP=COLOR?white_bishop_id:black_bishop_id;
  constexpr int ROOK=COLOR?white_rook_id:black_rook_id;
  constexpr int QUEEN=COLOR?white_queen_id:black_queen_id;
  constexpr int KING=COLOR?white_king_id:black_king_id;

  BitBoard bb=cp.piece(PAWN);
  while(bb){
    Square s=bb.get_reset_square().get_unique_square();
    m.piece_id(PAWN);
    m.from(s);
    m.code(normal_move);
    // normal captures:
    BitBoard bb2=m_moves[s.id()];
    bb2&=COLOR?~rank8:~rank1;
    bb2&=cp.pieces();
    bb2&=COLOR?~cp.white_pieces():~cp.black_pieces();
    while(bb2){
      Square s2=bb2.get_reset_square().get_unique_square();
      m.capture_id(cp.get_piece_id_from(s2));
      m.to(s2);
      set_checkstatus<COLOR>(m);
      if(is_legal_pawnmove(m)){r.push_back(m);}
    }
    // en passant:
    bb2=cp.ep().bitboard(); 
    while(bb2){
      Square s2=bb2.get_reset_square().get_unique_square();
      if((COLOR==white && (s2.id()-s.id()==7 || s2.id()-s.id()==9)) || (COLOR==black && (s2.id()-s.id()==-7 || s2.id()-s.id()==-9))){
	if(abs(s.file_number()-s2.file_number())<2){
	  m.capture_id(COLOR?black_pawn_id:white_pawn_id);
	  m.to(s2);
	  m.code(en_passant);
	  set_checkstatus<COLOR>(m);
	  if(is_legal_pawnmove(m)){r.push_back(m);}
	}
      }
    }
    // promotions:
    bb2=m_moves[s.id()];
    bb2&=COLOR?rank8:rank1;
    bb2&=COLOR?~cp.white_pieces():~cp.black_pieces();
    while(bb2){
      Square s2=bb2.get_reset_square().get_unique_square();
      m.capture_id(cp.get_piece_id_from(s2));
      m.to(s2);
      m.code(promo_queen);
      set_checkstatus<COLOR>(m);
      if(is_legal_pawnmove(m)){r.push_back(m);}
      m.code(promo_rook);
      set_checkstatus<COLOR>(m);
      if(is_legal_pawnmove(m)){r.push_back(m);}
      m.code(promo_bishop);
      set_checkstatus<COLOR>(m);
      if(is_legal_pawnmove(m)){r.push_back(m);}
      m.code(promo_knight);
      set_checkstatus<COLOR>(m);
      if(is_legal_pawnmove(m)){r.push_back(m);}
    }
 }

  bb=cp.piece(KNIGHT);
  while(bb){
    Square s=bb.get_reset_square().get_unique_square();
    m.piece_id(KNIGHT);
    m.from(s);
    m.code(normal_move);
    BitBoard bb2=m_moves[s.id()];
    bb2&=COLOR?~cp.white_pieces():~cp.black_pieces();
    bb2&=cp.pieces();
    while(bb2){
      Square s2=bb2.get_reset_square().get_unique_square();
      m.capture_id(cp.get_piece_id_from(s2));
      m.to(s2);
      set_checkstatus<COLOR>(m);
      if(is_legal(m)){r.push_back(m);}
    }
  }

  bb=cp.piece(BISHOP);
  while(bb){
    Square s=bb.get_reset_square().get_unique_square();
    m.piece_id(BISHOP);
    m.from(s);
    m.code(normal_move);
    BitBoard bb2=m_moves[s.id()];
    bb2&=COLOR?~cp.white_pieces():~cp.black_pieces();
    bb2&=cp.pieces();
    while(bb2){
      Square s2=bb2.get_reset_square().get_unique_square();
      m.capture_id(cp.get_piece_id_from(s2));
      m.to(s2);
      set_checkstatus<COLOR>(m);
      if(is_legal(m)){r.push_back(m);}
    }
  }

  bb=cp.piece(ROOK);
  while(bb){
    Square s=bb.get_reset_square().get_unique_square();
    m.piece_id(ROOK);
    m.from(s);
    m.code(normal_move);
    BitBoard bb2=m_moves[s.id()];
    bb2&=COLOR?~cp.white_pieces():~cp.black_pieces();
    bb2&=cp.pieces();
    while(bb2){
      Square s2=bb2.get_reset_square().get_unique_square();
      m.capture_id(cp.get_piece_id_from(s2));
      m.to(s2);
      set_checkstatus<COLOR>(m);
      if(is_legal(m)){r.push_back(m);}
    }
  }

  bb=cp.piece(QUEEN);
  while(bb){
    Square s=bb.get_reset_square().get_unique_square();
    m.piece_id(QUEEN);
    m.from(s);
    m.code(normal_move);
    BitBoard bb2=m_moves[s.id()];
    bb2&=COLOR?~cp.white_pieces():~cp.black_pieces();
    bb2&=cp.pieces();
    while(bb2){
      Square s2=bb2.get_reset_square().get_unique_square();
      m.capture_id(cp.get_piece_id_from(s2));
      m.to(s2);
      set_checkstatus<COLOR>(m);
      if(is_legal(m)){r.push_back(m);}
    }
  }

  bb=cp.piece(KING);
  while(bb){
    Square s=bb.get_reset_square().get_unique_square();
    m.piece_id(KING);
    m.from(s);
    m.code(normal_move);
    BitBoard bb2=m_moves[s.id()];
    bb2&=COLOR?~cp.white_pieces():~cp.black_pieces();
    bb2&=cp.pieces();
    while(bb2){
      Square s2=bb2.get_reset_square().get_unique_square();
      m.capture_id(cp.get_piece_id_from(s2));
      m.to(s2);
      set_checkstatus<COLOR>(m);
      if(is_legal_kingmove<COLOR>(m)){r.push_back(m);}
    }
  }


  //---------------------- MVV/LVA -----------------------
  for(unsigned i=1;i<r.size();i++){
    Move x=r[i];
    int j=i-1;
    while(j>=0 && r[j].capture().abs_value()<x.capture().abs_value()){
      r[j+1]=r[j];
      j--;
    }
    r[j+1]=x;
  }
  
  
}

template<int COLOR>
void MoveGenerator::generate_quiet_moves(vector<Move>& r)
{
  Move m;
  constexpr int PAWN=COLOR?white_pawn_id:black_pawn_id;
  constexpr int KNIGHT=COLOR?white_knight_id:black_knight_id;
  constexpr int BISHOP=COLOR?white_bishop_id:black_bishop_id;
  constexpr int ROOK=COLOR?white_rook_id:black_rook_id;
  constexpr int QUEEN=COLOR?white_queen_id:black_queen_id;
  constexpr int KING=COLOR?white_king_id:black_king_id;

  BitBoard bb=cp.piece(KNIGHT);
  while(bb){
    Square s=bb.get_reset_square().get_unique_square();
    m.piece_id(KNIGHT);
    m.from(s);
    m.code(normal_move);
    BitBoard bb2=m_moves[s.id()];
    bb2&=~cp.pieces();
    while(bb2){
      Square s2=bb2.get_reset_square().get_unique_square();
      m.capture_id(cp.get_piece_id_from(s2));
      m.to(s2);
      set_checkstatus<COLOR>(m);
      if(is_legal(m)){r.push_back(m);}
    }
  }

  bb=cp.piece(BISHOP);
  while(bb){
    Square s=bb.get_reset_square().get_unique_square();
    m.piece_id(BISHOP);
    m.from(s);
    m.code(normal_move);
    BitBoard bb2=m_moves[s.id()];
    bb2&=~cp.pieces();
    while(bb2){
      Square s2=bb2.get_reset_square().get_unique_square();
      m.capture_id(cp.get_piece_id_from(s2));
      m.to(s2);
      set_checkstatus<COLOR>(m);
      if(is_legal(m)){r.push_back(m);}
    }
  }

  bb=cp.piece(ROOK);
  while(bb){
    Square s=bb.get_reset_square().get_unique_square();
    m.piece_id(ROOK);
    m.from(s);
    m.code(normal_move);
    BitBoard bb2=m_moves[s.id()];
    bb2&=~cp.pieces();
    while(bb2){
      Square s2=bb2.get_reset_square().get_unique_square();
      m.capture_id(cp.get_piece_id_from(s2));
      m.to(s2);
      set_checkstatus<COLOR>(m);
      if(is_legal(m)){r.push_back(m);}
    }
  }

  bb=cp.piece(QUEEN);
  while(bb){
    Square s=bb.get_reset_square().get_unique_square();
    m.piece_id(QUEEN);
    m.from(s);
    m.code(normal_move);
    BitBoard bb2=m_moves[s.id()];
    bb2&=~cp.pieces();
    while(bb2){
      Square s2=bb2.get_reset_square().get_unique_square();
      m.capture_id(cp.get_piece_id_from(s2));
      m.to(s2);
      set_checkstatus<COLOR>(m);
      if(is_legal(m)){r.push_back(m);}
    }
  }

  bb=cp.piece(PAWN);
  while(bb){
    Square s=bb.get_reset_square().get_unique_square();
    m.piece_id(PAWN);
    m.from(s);
    m.code(normal_move);
    BitBoard bb2=m_moves[s.id()];
    bb2&=COLOR?~rank8:~rank1;
    bb2&=~cp.pieces();
    while(bb2){
      Square s2=bb2.get_reset_square().get_unique_square();
      m.capture_id(cp.get_piece_id_from(s2));
      m.to(s2);
      set_checkstatus<COLOR>(m);
      if(is_legal(m)){r.push_back(m);}
    }
  }
  
  bb=cp.piece(KING);
  while(bb){
    Square s=bb.get_reset_square().get_unique_square();
    m.piece_id(KING);
    m.from(s);
    m.code(normal_move);
    BitBoard bb2=m_moves[s.id()];
    bb2&=~cp.pieces();
    while(bb2){
      Square s2=bb2.get_reset_square().get_unique_square();
      m.capture_id(cp.get_piece_id_from(s2));
      m.to(s2);
      set_checkstatus<COLOR>(m);
      if(is_legal_kingmove<COLOR>(m)){r.push_back(m);}
    }
  }

  
  {//-------------------- castling move: ----------------------------------------------------------
    Square kingsfield=COLOR==white?cp.white_king().get_unique_square():cp.black_king().get_unique_square();
    Square rooksfield;
    bool castling_ok{true};
    uint16_t castles=COLOR==white?cp.castlingrights()&0x00FF:cp.castlingrights()&0xFF00;

    while(castles){
      castling_ok=true;
      uint16_t f=castles & (castles ^(castles - 1));
      castles&=~f;
      switch(f){
      case 0x0001:rooksfield=a1;break;
      case 0x0002:rooksfield=b1;break;
      case 0x0004:rooksfield=c1;break;
      case 0x0008:rooksfield=d1;break;
      case 0x0010:rooksfield=e1;break;
      case 0x0020:rooksfield=f1;break;
      case 0x0040:rooksfield=g1;break;
      case 0x0080:rooksfield=h1;break;
      case 0x0100:rooksfield=a8;break;
      case 0x0200:rooksfield=b8;break;
      case 0x0400:rooksfield=c8;break;
      case 0x0800:rooksfield=d8;break;
      case 0x1000:rooksfield=e8;break;
      case 0x2000:rooksfield=f8;break;
      case 0x4000:rooksfield=g8;break;
      case 0x8000:rooksfield=h8;break;
      }
      Square kingsfield2,rooksfield2;
      if(rooksfield>kingsfield){
	kingsfield2=COLOR==white?g1:g8;
	rooksfield2=COLOR==white?f1:f8;
      }else{
	kingsfield2=COLOR==white?c1:c8;
	rooksfield2=COLOR==white?d1:d8;      
      }
      if(m_con.closed_line_segment(kingsfield,kingsfield2) & ~m_ownkingsfield & ~rooksfield.bitboard() & m_pieces){castling_ok=false;}
      if(m_con.closed_line_segment(rooksfield,rooksfield2) & ~m_ownkingsfield & ~rooksfield.bitboard() & m_pieces){castling_ok=false;}
    
      BitBoard bb{m_con.closed_line_segment(kingsfield,kingsfield2)};
      while(bb){
	BitBoard sq=bb.get_reset_square();
	if(COLOR==white?cp.black_attacks(sq):cp.white_attacks(sq)){
	  castling_ok=false;
	  break;
	}
      }

      BitBoard rookattacks=m_atta.rook(kingsfield2.bitboard(),m_pieces & ~m_ownkingsfield & ~rooksfield.bitboard());
      if(rookattacks & (COLOR==white?cp.black_queen():cp.white_queen())){castling_ok=false;}
      if(rookattacks & (COLOR==white?cp.black_rook():cp.white_rook())){castling_ok=false;}
    
      if(castling_ok){
	m.to(rooksfield.id());
	m.code(castling);
	m.piece(COLOR==white?white_king_id:black_king_id);
	m.capture(0);
	set_checkstatus<COLOR>(m);
	r.push_back(m);
      }
    }
  }
}
