/*

Copyright (C) 2018 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#ifndef MOVEGENERATOR_H
#define MOVEGENERATOR_H

#include"../board/cposition.h"
#include"../common/attacks.h"
#include"../movegen/mlist.h"


class MoveGenerator
{
 private:
  ChessPosition& cp;
  const Connections& m_con;
  const Attacks& m_atta;
  array<BitBoard,64> m_moves;
  vector<Move> m_movelist;
  
  BitBoard m_pieces,m_pinned_pieces,m_discovering_pieces,m_own_pieces,m_opponent_pieces;
  BitBoard m_ownkingsfield,m_opponentkingsfield;
  BitBoard m_rookattacks,m_bishopattacks,m_knightattacks,m_pawnattacks;
  
  vector<Square> m_king_attackers;
  bool is_legal(const Move&)const;
  bool is_legal_pawnmove(const Move&)const;
  template<int> bool is_legal_kingmove(const Move&)const;
  template<int> void set_checkstatus(Move&);
  template<int> bool discovered_check(const BitBoard&,const BitBoard&);
  
 public:
  MoveGenerator(ChessPosition&);
  template<class T> void generate_moves(T&);
  template<class T> void generate_quiescence_moves(T&);

  template<int> void generate_movetable();
  template<int> void generate_captures(vector<Move>&);
  template<int> void generate_quiet_moves(vector<Move>&);
  const array<BitBoard,64>& movetable()const{return m_moves;};
  const vector<Move>& movelist()const{return m_movelist;}
};


#endif
