/*

Copyright (C) 2018 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include<cassert>
#include"movelist.h"


MoveList::MoveList(XMoveList& xml) : m_xml(xml)
{
  m_xml.increment();
}

MoveList::~MoveList()
{
  m_xml.decrement();
}

bool MoveList::operator==(const MoveList& ml)const
{
  return m_xml[m_xml.current()]==ml.m_xml[m_xml.current()];
}

Move& MoveList::operator[](int i)
{
  return m_xml[i];
}

void MoveList::set_chessposition(ChessPosition& cp)
{
  m_xml.set_chessposition(cp);
}

void MoveList::set_analysis(Analysis* pa)
{
  m_xml.set_analysis(pa);
}

void MoveList::set_searchmoves(const vector<Move>& vsm)
{
  m_xml.set_searchmoves(vsm);
}

void MoveList::set_hashtable_move(const Move& mh)
{
  m_xml.set_hashtable_move(mh);
}

void MoveList::set_killers(const Move& ka,const Move& kb)
{
  m_xml.set_killers(ka,kb);
}

void MoveList::set_killers_2(const Move& ka,const Move& kb)
{
  m_xml.set_killers_2(ka,kb);
}

void MoveList::set_log(bool x)
{
  m_xml.set_log(x);
}

void MoveList::set_xcodes()
{
  m_xml.set_xcodes();
}

void MoveList::sort()
{
  m_xml.sort();
}

void MoveList::add(const Move& m)
{
  m_xml.add(m);
}

void MoveList::remove(const Move& m)
{
  m_xml.remove(m);
}

void MoveList::clear()
{
  m_xml.clear();
}

const Move& MoveList::first()
{
  return m_xml.first();
}

const Move& MoveList::next()
{
  return m_xml.next();
}

Move& MoveList::back()
{
  return m_xml.back();
}

void MoveList::pop_back()
{
  m_xml.pop_back();
}

void MoveList::push_back(const Move& x)
{
  m_xml.push_back(x);
}

void MoveList::insert(vector<Move>::iterator it_begin,vector<Move>::iterator it_end)
{
  m_xml.insert(it_begin,it_end);
}

unsigned int MoveList::size()const
{
  return m_xml.size();
}

string MoveList::dump()const
{
  return m_xml.dump();
}
