/*

Copyright (C) 2018 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include"duration.h"
using std::nano;
using std::milli;
using std::chrono::duration;

Duration::Duration(const Duration& x)
{
  m_d=x.m_d;
}

Duration::Duration(const std::chrono::duration<double,std::ratio<1>>& x)
{
  m_d=x;
}

Duration& Duration::operator=(const Duration& x)
{
  m_d=x.m_d;
  return *this;
}

Duration& Duration::operator+=(const Duration& x)
{
  m_d+=x.m_d;
  return *this;
}

Duration& Duration::operator-=(const Duration& x)
{
  m_d-=x.m_d;
  return *this;
}

bool Duration::operator==(const Duration& x)const
{
  return m_d==x.m_d;
}

bool Duration::operator!=(const Duration& x)const
{
  return m_d!=x.m_d;
}

long long Duration::nanoseconds()const
{
  duration<double,nano> d;
  d=m_d;
  return static_cast<long long>(d.count());
}

long long Duration::microseconds()const
{
  duration<double,micro> d;
  d=m_d;
  return static_cast<long long>(d.count());
}

long long Duration::milliseconds()const
{
  duration<double,milli> d;
  d=m_d;
  return static_cast<long long>(d.count());
}

long long Duration::centiseconds()const
{
  duration<double,milli> d;
  d=m_d;
  return static_cast<long long>(d.count()/10);
}

long long Duration::seconds()const
{
  duration<double,milli> d;
  d=m_d;
  return static_cast<long long>(d.count()/1000);
}

long long Duration::minutes()const
{
  duration<double,milli> d;
  d=m_d;
  return static_cast<long long>(d.count()/60000);
}

void Duration::nanoseconds(long long x)
{
  std::chrono::nanoseconds d{x};
  m_d=d;
}

void Duration::milliseconds(long long x)
{
  std::chrono::milliseconds d{x};
  m_d=d;
}

void Duration::centiseconds(long long x)
{
  std::chrono::milliseconds d{x*10};
  m_d=d;
}

void Duration::minutes(long long x)
{
  std::chrono::minutes d{x};
  m_d=d;
}

void Duration::seconds(float x)
{
  std::chrono::milliseconds d{static_cast<long long>(x*1000.0)};
  m_d=d;
}


//---------------- help functions: -----------------------------
Duration operator-(Duration x,Duration y){return x-=y;}
Duration operator+(Duration x,Duration y){return x+=y;}
