/*

Copyright (C) 2018 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include"multipv.h"
#include<algorithm>
#include<cassert>



MultiPV::MultiPV(const MultiPV& x)
{
  m_number=x.m_number;
  m_version=x.m_version;
  m_multipv=x.m_multipv;
}

MultiPV& MultiPV::operator=(const MultiPV& x)
{  
  m_number=x.m_number;
  m_version++;
  m_multipv=x.m_multipv;
  return *this;
}

PrincipalVariation& MultiPV::operator[](int x)
{
  return m_multipv[x];
}

const PrincipalVariation& MultiPV::operator[](int x)const
{
  return m_multipv[x];
}

void MultiPV::clear()
{
  m_multipv.clear();
}

void MultiPV::set_number(int x)
{
  m_number=x;
}

int MultiPV::number()const
{
  return m_number;
}

int MultiPV::size()const
{
  return m_multipv.size();
}

int MultiPV::version()const
{
  return m_version;
}

void MultiPV::add(PrincipalVariation pv)
{
  if(pv.size()<=0){return;}
  for(size_t i=0;i<m_multipv.size();i++){
    if(pv[0]==m_multipv[i][0]){
      m_multipv[i]=m_multipv.back();
      m_multipv.pop_back();
      break;
    }
  }
  
  m_multipv.push_back(pv);
  for(size_t i=0;i<m_multipv.size();i++){
    size_t best_j=i;
    for(size_t j=i+1;j<m_multipv.size();j++){
      if(m_multipv[j]>m_multipv[i]){
	best_j=j;
      }
    }
    if(best_j>i){
      swap(m_multipv[i],m_multipv[best_j]);
    }
  }
  
  while(m_multipv.size()>m_number)
    m_multipv.pop_back();
  m_version++;
}

void MultiPV::randomize(int handicap)
{
  std::random_device rd;
  int nbr=rd();
  nbr=nbr<0?-nbr:nbr;
  
  //----------------------- sort best pv first: --------------------------------
  for(size_t i=0;i<m_multipv.size();i++){
    unsigned int best_j=i;
    for(size_t j=i;j<m_multipv.size();j++){
      if(m_multipv[best_j].score()<m_multipv[j].score()){
	best_j=j;
      }
    }
    if(best_j>i){
      swap(m_multipv[best_j],m_multipv[i]);
    }
  }
  //----------------------------------------------------------------------------

  int threshold=m_multipv[0].score()-handicap;
  int end{0};
  for(size_t i=0;i<m_multipv.size();i++){
    if(m_multipv[i].score()>=threshold){end=i+1;}
  }

  nbr%=end;
  if(nbr>0)
    swap(m_multipv[0],m_multipv[nbr]);
}

vector<PrincipalVariation>::iterator MultiPV::begin()
{
  return m_multipv.begin();
}

vector<PrincipalVariation>::iterator MultiPV::end()
{
  return m_multipv.end();
}

vector<PrincipalVariation>::const_iterator MultiPV::cbegin()const
{
  return m_multipv.begin();
}

vector<PrincipalVariation>::const_iterator MultiPV::cend()const
{
  return m_multipv.end();
}
