/*

Copyright (C) 2018 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include<stdexcept>
#include<sstream>
#include"principal.h"
using std::logic_error;
using std::ostringstream;


PrincipalVariation::PrincipalVariation(const PrincipalVariation& x)
{
  m_version=x.m_version;
  int i=x.m_size;
  m_size=i;
  i=x.m_score;
  m_score=i;
  i=x.m_depth;
  m_depth=i;
  m_pv=x.m_pv;
}

PrincipalVariation& PrincipalVariation::operator=(const PrincipalVariation& x)
{
  m_version++;
  int i=x.m_size;
  m_size=i;
  i=x.m_score;
  m_score=i;
  i=x.m_depth;
  m_depth=i;
  m_pv=x.m_pv;
  return *this;
}

bool PrincipalVariation::operator==(const PrincipalVariation& x)const
{
  return m_pv==x.m_pv;
}

bool PrincipalVariation::operator!=(const PrincipalVariation& x)const
{
  return m_pv!=x.m_pv;
}

bool PrincipalVariation::operator>(const PrincipalVariation& x)const
{
  if(m_depth>x.m_depth){
    return true;
  }else if(m_depth<x.m_depth){
    return false;
  }else if(m_score>x.m_score){
    return true;
  }else{
    return false;
  }
}

int PrincipalVariation::size()const
{
  return m_size;
}

int PrincipalVariation::depth()const
{
  return m_depth;
}

int PrincipalVariation::score()const
{
  return m_score;
}

void PrincipalVariation::set_depth(int x)
{
  m_depth=x;
}

void PrincipalVariation::set_score(int x)
{
  m_score=x;
}

int PrincipalVariation::version()const
{
  return m_version;
}

string PrincipalVariation::str()const
{
  ostringstream oss;
  for(auto iter=cbegin();iter!=cend();iter++){
    oss << iter->algebraic() << " ";
  }
  oss << std::endl;
  return oss.str();
}

void PrincipalVariation::clear()
{
  m_size=0;
  m_score=0;
}

Move& PrincipalVariation::operator[](int i)
{
  if(i>m_size){throw logic_error("PrincipalVariation::operator[]() parameter out of range");}
  if(i==m_size){++m_size;}
  return m_pv[i];
}

const Move& PrincipalVariation::operator[](int i)const
{
  if(i>=m_size){throw logic_error("PrincipalVariation::operator[]() parameter out of range");}  
  return m_pv[i];;
}

PrincipalVariation operator+(Move m,PrincipalVariation& x)
{
  PrincipalVariation r;
  r[0]=m;
  for(int i=0;i<x.size();++i){
    r[i+1]=x[i];
  }
  return r;
}

const Move* PrincipalVariation::cbegin()const
{
  return &m_pv[0];
}

const Move* PrincipalVariation::cend()const
{
  return cbegin()+m_size;
}

Move* PrincipalVariation::begin()
{
  return &m_pv[0];
}

Move* PrincipalVariation::end()
{
  return begin()+m_size;
}
