/*

Copyright (C) 2018 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#ifndef PRINCIPAL_H
#define PRINCIPAL_H

#include<string>
#include<array>
#include<iterator>
#include<atomic>
#include<mutex>
#include"../common/numerals.h"
using std::array;
using std::atomic;
using std::mutex;
using std::string;

class PrincipalVariation
{
 private:
  atomic<int> m_size{0};
  atomic<int> m_score{0};
  atomic<int> m_depth{0};
  array<Move,max_search_depth> m_pv;
  int m_version{0};

 public:
  PrincipalVariation()=default;
  PrincipalVariation(const PrincipalVariation&);
  PrincipalVariation& operator=(const PrincipalVariation&);
  bool operator==(const PrincipalVariation&)const;
  bool operator!=(const PrincipalVariation&)const;
  bool operator>(const PrincipalVariation&)const;
  Move& operator[](int);
  const Move& operator[](int)const;
  void clear();
  int size()const;
  int depth()const;
  int score()const;
  void set_depth(int);
  void set_score(int);
  int version()const;
  string str()const;
  Move* begin();
  Move* end();
  const Move* cbegin()const;
  const Move* cend()const;
};

PrincipalVariation operator+(Move,PrincipalVariation&);


#endif
