/*

Copyright (C) 2018 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#ifndef THREADPOOL_H
#define THREADPOOL_H

#include"analyze.h"
#include<thread>
#include<vector>
#include<string>
using std::vector;
using std::string;
using std::thread;


class ThreadPool
{
 private:
  TranspositionTable& m_tt;
  unsigned m_number_of_threads{0};
  vector<ChessPosition> m_cp;
  vector<Analysis*> m_pana;
  vector<thread> m_thread;
  mutable vector<Timer> m_timer;
  vector<bool> m_busy;
  unsigned m_running_threads{0};
  atomic<bool> m_free_thread{true};
  mutable mutex m_mutex_threadpool;

 public:
  ThreadPool(int,TranspositionTable&);
  ThreadPool(const ThreadPool&);
  ~ThreadPool();
  Analysis* operator[](int)const;
  void clear();
  int size()const;
  void set_size(const int&);
  long long get_nodes()const;
  void clear_nodes();
  bool open_new_thread(int&);
  string state()const;
  void nw_search(const int&,float,int,int,int,int,int,const Analysis&);
  void result(const int&,int&,int&);
  bool is_spinning(const int&)const;
  void stop_spinning();
};


#endif
