/*

Copyright (C) 2018 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include"timemanager.h"


TimeManager::TimeManager()
{
  reset();
}

void TimeManager::reset()
{
  m_tim.reset();
  m_movetime=999999999;
  m_wtime=m_movetime;
  m_btime=m_movetime;
  m_winc=0;
  m_binc=0;
  m_movestogo=999;
  m_nodes=(1ull<<60);
  m_mate=999999;
  m_infinite=false;
  m_pondering=false;
}

void TimeManager::start()
{
  m_tim.reset();
  m_tim.start();
}

bool TimeManager::time_to_move(long long nodes)
{
  int t=m_tim.total_time().milliseconds();
  int etime,einc;
  if(m_infinite){return false;}
  if(m_pondering){return false;}
  if(nodes>m_nodes){return true;}
  if(m_movetime>0 && t>=m_movetime-500){return true;}  
  bool r=false;
  if(m_movetime==999999999){
    if(m_white_to_move){etime=m_wtime;einc=m_winc;}
    else{etime=m_btime;einc=m_binc;}
    etime+=einc*(m_movestogo);
    if(t>(etime/(m_movestogo+1))){r=true;}
  }
  return r;
}

int TimeManager::remaining_time()
{
  int r;
  int t=m_tim.total_time().milliseconds();
  int etime{0};
  if(m_movetime==999999999){
    if(m_white_to_move){
      etime=m_wtime;
    }else{
      etime=m_btime;
    }
    r=etime-t;
  }else{
    r=m_movetime-t;
  }
  return r;
}

void TimeManager::set_wtime(int x)
{
  m_wtime=x;
}

void TimeManager::set_btime(int x)
{
  m_btime=x;
}

void TimeManager::set_winc(int x)
{
  m_winc=x;
}

void TimeManager::set_binc(int x)
{
  m_binc=x;
}

void TimeManager::set_movestogo(int x)
{
  m_movestogo=x;
  if(m_movestogo<=0){m_movestogo=30;}  // sudden death game
}

void TimeManager::set_nodes(long long x)
{
  m_nodes=x;
}

void TimeManager::set_mate(int x)
{
  m_mate=x;
}

void TimeManager::set_movetime(long long x)
{
  m_movetime=x;
}

void TimeManager::set_infinite(bool x)
{
  m_infinite=x;
}

void TimeManager::set_pondering(bool x)
{
  m_pondering=x;
}

void TimeManager::set_white_to_move(bool x)
{
  m_white_to_move=x;
}
