/*

Copyright (C) 2018 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include"drawing.h"
#include"promochoicedialog.h"
#include<cairomm/context.h>
#include<gdkmm/general.h> // set_source_pixbuf()
#include<glibmm/fileutils.h>
#include<iostream>
#include<iomanip>
#include<fstream>
#include<string>
#include<stdexcept>
using std::ifstream;
using std::string;
using std::cout;
using std::endl;
using std::hex;
using std::logic_error;



Drawing::Drawing()
{
  set_size_request(480,480);
  m_square_height=m_square_width=60;
  v_delta=h_delta=60;
  
  for(int i=0;i<8;i++){
    for(int j=0;j<8;j++){
      if((i+j)%2==0){empty_board.push_back(bsquare);}
      else{empty_board.push_back(wsquare);}
      board.push_back(empty_board.back());
    }
  }
  load_images(m_square_width,m_square_height);

}


Drawing::~Drawing()
{
}


bool Drawing::on_draw(const Cairo::RefPtr<Cairo::Context>& cr)
{
  Gtk::Allocation allocation = get_allocation();
  const int width = allocation.get_width();
  const int height = allocation.get_height();
  h_delta=width/8;
  v_delta=height/8;
  if(h_delta!=m_square_width || v_delta!=m_square_height){
    m_square_width=h_delta;
    m_square_height=v_delta;
    load_images(m_square_width,m_square_height);
  }
  
  for(int i=0;i<8;i++){
    for(int j=0;j<8;j++){
      if(board[(7-i)*8+j]!=wsquare && board[(7-i)*8+j]!=bsquare){
	cr->rectangle(j*h_delta,i*v_delta,(j+1)*h_delta,(i+1)*v_delta);
	Gdk::Cairo::set_source_pixbuf(cr,m_image.at(empty_board.at((7-i)*8+j)),j*h_delta,i*v_delta);
	cr->fill();
	cr->rectangle(j*h_delta,i*v_delta,(j+1)*h_delta,(i+1)*v_delta);
	Gdk::Cairo::set_source_pixbuf(cr,m_image.at(board.at((7-i)*8+j)),j*h_delta,i*v_delta);
	cr->fill();
      }
      else{
	cr->rectangle(j*h_delta,i*v_delta,(j+1)*h_delta,(i+1)*v_delta);
	Gdk::Cairo::set_source_pixbuf(cr,m_image.at(empty_board.at((7-i)*8+j)),j*h_delta,i*v_delta);
	cr->fill();
      }
    }
  }
  
  if(dragging){
    cr->rectangle(x_coordinate-h_delta/2,x_coordinate-v_delta/2,
		  x_coordinate+h_delta/2,y_coordinate+v_delta/2);
    Gdk::Cairo::set_source_pixbuf(cr,m_image.at(dragged_piece),x_coordinate,y_coordinate);  
    cr->fill();
  }
  
  cr->paint();
  return true;
}

void Drawing::load_images(int width,int height)
{
  string prefix;
  ifstream ifs("galjoen.files/png/wood_l.png");
  if(ifs.good()){prefix.clear();}
  else{
    ifs.close();
    ifs.open("/usr/local/games/galjoen.files/png/wood_l.png");
    if(ifs.good()){
      prefix="/usr/local/games/";
    }
    else{
      prefix="/usr/games/";
    }
  }
  
  try{  
    m_image.clear();
    m_image.push_back(Gdk::Pixbuf::create_from_file(prefix+
						  "galjoen.files/png/wood_l.png",width,height,false));
    m_image.push_back(Gdk::Pixbuf::create_from_file(prefix+
						  "galjoen.files/svg/WhitePawn.svg",width,height,false));
    m_image.push_back(Gdk::Pixbuf::create_from_file(prefix+
						  "galjoen.files/svg/WhiteKnight.svg",width,height,false)); 
    m_image.push_back(Gdk::Pixbuf::create_from_file(prefix+
						  "galjoen.files/svg/WhiteBishop.svg",width,height,false));
    m_image.push_back(Gdk::Pixbuf::create_from_file(prefix+
						  "galjoen.files/svg/WhiteRook.svg",width,height,false));
    m_image.push_back(Gdk::Pixbuf::create_from_file(prefix+
						  "galjoen.files/svg/WhiteQueen.svg",width,height,false));
    m_image.push_back(Gdk::Pixbuf::create_from_file(prefix+
						  "galjoen.files/svg/WhiteKing.svg",width,height,false));
    
    m_image.push_back(Gdk::Pixbuf::create_from_file(prefix+
						  "galjoen.files/png/wood_d.png",width,height,false));
    m_image.push_back(Gdk::Pixbuf::create_from_file(prefix+
						  "galjoen.files/svg/BlackPawn.svg",width,height,false));
    m_image.push_back(Gdk::Pixbuf::create_from_file(prefix+
						  "galjoen.files/svg/BlackKnight.svg",width,height,false));
    m_image.push_back(Gdk::Pixbuf::create_from_file(prefix+
						  "galjoen.files/svg/BlackBishop.svg",width,height,false));
    m_image.push_back(Gdk::Pixbuf::create_from_file(prefix+
						  "galjoen.files/svg/BlackRook.svg",width,height,false));
    m_image.push_back(Gdk::Pixbuf::create_from_file(prefix+
						  "galjoen.files/svg/BlackQueen.svg",width,height,false));
    m_image.push_back(Gdk::Pixbuf::create_from_file(prefix+
						  "galjoen.files/svg/BlackKing.svg",width,height,false));
  }
  catch(...){
    std::cout << "problem" << std::endl;
    std::cout << "prefix=" << prefix << std::endl;
  }
}

vector<Glib::RefPtr<Gdk::Pixbuf>> Drawing::get_images()const
{
  return m_image;
}

void Drawing::initialize(const vector<char>& v)
{
  board=empty_board;
  for(int i=0;i<64;i++){
    if(v[i]!='?'){set_piece_on(v[i],i);}
  }
  queue_draw();
}

void Drawing::set_parent_window(Gtk::Window* ppw)
{
  p_parent_window=ppw;
}

void Drawing::set_piece_on(char p,int s)
{
  switch(p)
    {
    case 'P':board[s]=wpawn;break;
    case 'N':board[s]=wknight;break;
    case 'B':board[s]=wbishop;break;
    case 'R':board[s]=wrook;break;
    case 'Q':board[s]=wqueen;break;
    case 'K':board[s]=wking;break;
    case 'p':board[s]=bpawn;break;
    case 'n':board[s]=bknight;break;
    case 'b':board[s]=bbishop;break;
    case 'r':board[s]=brook;break;
    case 'q':board[s]=bqueen;break;
    case 'k':board[s]=bking;break;
    default:std::cout << "unknown piece: 0x" << hex << int(p) << std::endl;throw logic_error("stop.."); 
    }
}

void Drawing::remove_piece_from(int s)
{
  board[s]=empty_board[s];
}

void Drawing::button1_pressed(int x,int y)
{
  x/=h_delta;
  y/=v_delta;
  int i=7-y;
  int j=x;
  int sq=8*i+j;
  if(board[sq]!=wsquare && board[sq]!=bsquare){
    dragging=true;
    original_square=sq;
    dragged_piece=board[sq];
    board[sq]=empty_board[sq];
  }
  else{
    dragging=false;
  }
}

pair<int,int> Drawing::button1_released(int x,int y)
{
  pair<int,int> r;
  x/=h_delta;
  y/=v_delta;
  int i=7-y;
  int j=x;
  int sq=8*i+j;
  r.first=r.second=sq;
  if(dragging){
    board[original_square]=empty_board[original_square];
    board[sq]=dragged_piece;
    dragging=false;
    r.first=original_square;
    queue_draw();

    if(r.second>=0 && r.second<8 && dragged_piece==bpawn){
      PromoChoiceDialog pcd(p_parent_window,m_image[bqueen],m_image[brook],m_image[bbishop],m_image[bknight]);
      r.second|=pcd.get_choice()<<6;
    }else if(r.second>=56 && r.second<64 && dragged_piece==wpawn){
      PromoChoiceDialog pcd(p_parent_window,m_image[wqueen],m_image[wrook],m_image[wbishop],m_image[wknight]);
      r.second|=pcd.get_choice()<<6;
    }
  }
  return r;
}

void Drawing::drag_piece_to(int x,int y)
{
  if(dragging){
    x_coordinate=x-h_delta/2;
    y_coordinate=y-v_delta/2;
    queue_draw();
  }
}

