/*

Copyright (C) 2018 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#ifndef GTKMM_DRAWING_H
#define GTKMM_DRAWING_H

#include<vector>
#include<utility>
#include<gtkmm/drawingarea.h>
#include<gtkmm/window.h>
#include<gdkmm/pixbuf.h>
using std::vector;
using std::pair;


class Drawing : public Gtk::DrawingArea
{
 public:
  Drawing();
  virtual ~Drawing();

  void set_parent_window(Gtk::Window*);
  void initialize(const vector<char>&);
  void set_piece_on(char,int);
  void remove_piece_from(int);

  void button1_pressed(int,int);
  pair<int,int> button1_released(int,int);
  void drag_piece_to(int,int);
  vector<Glib::RefPtr<Gdk::Pixbuf>> get_images()const;
  
 protected:
  //Override default signal handlers:
  virtual bool on_draw(const Cairo::RefPtr<Cairo::Context>& cr);

 private:
  Gtk::Window* p_parent_window;
  vector<Glib::RefPtr<Gdk::Pixbuf>> m_image;

  enum piece_t {wsquare,wpawn,wknight,wbishop,wrook,wqueen,wking,
		bsquare,bpawn,bknight,bbishop,brook,bqueen,bking};  
  int h_delta,v_delta;
  int m_square_width,m_square_height;


  vector<piece_t> empty_board;
  vector<piece_t> board;
  void load_images(int,int);

  // drag piece:
  bool dragging;
  piece_t dragged_piece;
  int original_square;
  int x_coordinate, y_coordinate;
};



#endif // GTKMM_DRAWING_H

