/*

Copyright (C) 2018 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include"engio.h"
#include<thread>
#include<string>
#include<iostream>
#include<fstream>
#include<sstream>
#include<stdexcept>
#include<chrono>
using namespace std::chrono;
using std::ofstream;
using std::ostringstream;
using std::istringstream;
using std::runtime_error;
using std::string;
using std::thread;
using std::cout;
using std::cerr;
using std::flush;
using std::endl;


Engio::Engio(Communicator& cc): m_com(cc)
{
  m_engine_filename.clear();
}

Engio::~Engio()
{
  if(m_log){m_ofs.close();}
}

void Engio::operator()()
{
  auto read_from_master=[&]()
    {
      const int buffer_size{32768};
      char buffer[buffer_size];
      string s;
      while(!m_quit){
	s=m_com.from_interface();
	if(s.size()>0 && s.size()<buffer_size){
	  if(s.back()!='\n'){s+="\n";}
	  if(m_log){
	    time_point<system_clock> now=system_clock::now();
	    m_ofs << now.time_since_epoch().count() << "   engine <- " << s << flush;
	  }
	  for(size_t i=0;i<s.size();i++){buffer[i]=s[i];}
	  unsigned int size=write(m_in,buffer,s.size());
	  if(size!=s.size()){m_crashed=true;}
	}else if(s.size()>=buffer_size){
	  string E="Engio::operator() command from interface too big";
	  m_ofs << E << endl;
	  throw runtime_error(E);
	}
	std::this_thread::sleep_for(std::chrono::milliseconds(5));
      }
    };
  auto read_from_engine=[&]()
    {
      const int buffer_size{32768};
      char buffer[buffer_size];
      int size;
      string s;
      while(!m_quit){
	size=read(m_out,buffer,buffer_size);
	if(size<=0){m_crashed=true;}
	if(size>0){
	  for(int i=0;i<size;i++){s.push_back(buffer[i]);}
	}
	
	if(size>0 && (s.back()=='\n' || s.back()=='\r')){
	  istringstream iss(s);
	  string s1;
	  getline(iss,s1);
	  while(!iss.eof()){
	    m_com.to_interface(s1);
	    if(m_log){
	      time_point<system_clock> now=system_clock::now();
	      m_ofs << now.time_since_epoch().count() << "   engine -> " << s1 << endl;
	    }
	    getline(iss,s1);
	  }
	  s="";
	}
	std::this_thread::sleep_for(std::chrono::milliseconds(5));
      }
    };
  
  
  thread t1(read_from_master);
  thread t2(read_from_engine);
  t1.join();
  t2.join();
}

void Engio::set_engine(string x)
{
  m_engine_filename=x;

  gchar g_engine_filename[1000];
  for(size_t i=0;i<m_engine_filename.size() && i<999;i++){
    g_engine_filename[i]=m_engine_filename[i];
    g_engine_filename[i+1]=0;
  }
  gchar* argv[]={g_engine_filename,NULL};
  
  m_spawned=g_spawn_async_with_pipes ( NULL,
				     argv,
				     NULL,
				     G_SPAWN_DEFAULT,
				     NULL,
				     NULL,
				     &m_pid,
				     &m_in,
				     &m_out,
				     NULL,
				     NULL);    

  if(!m_spawned){
    std::cerr << "Engio::set_engine(" << x << ")  FAILED" << endl;
  }
}

void Engio::set_log(bool x)
{
  m_log=x;
  if(m_log){
    ostringstream oss;
    oss << "engine" << m_engine_id << ".log";
    log(oss.str());
  }
}

void Engio::log(string name)
{
  m_log_filename=name;
  m_log=true;
  if(m_ofs.good()){m_ofs.close();}
  m_ofs.open(m_log_filename,std::ios_base::out);
}

bool Engio::good()const
{
  return (m_spawned && !m_crashed);
}

bool Engio::crashed()const
{
  return m_crashed;
}

void Engio::quit()
{
  write(m_in,"quit\n",5);
  m_quit=true;
  g_spawn_close_pid(m_pid);
}

void Engio::set_engine_id(int x)
{
  m_engine_id=x;
}

int Engio::get_engine_id()const
{
  return m_engine_id;
}
