/*

Copyright (C) 2018 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include <iostream>
#include "fenlist.h"
#include "pgnfile.h"

FenList::FenList(Gtk::Window& x)
  : m_pw(x), m_VBox(Gtk::ORIENTATION_VERTICAL)
{
  //Add the TreeView, inside a ScrolledWindow, with the button underneath:
  m_ScrolledWindow.add(m_TreeView);

  //Only show the scrollbars when they are necessary:
  m_ScrolledWindow.set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
  m_VBox.pack_start(m_ScrolledWindow);

  //Create the Tree model:
  m_refTreeModel = Gtk::ListStore::create(m_Columns);
  m_TreeView.set_model(m_refTreeModel);

  //Add the TreeView's view columns:
  m_TreeView.append_column("ID", m_Columns.m_col_id);
  m_TreeView.append_column("Fen", m_Columns.m_col_fen);
}


FenList::~FenList()
{
}

void FenList::clear()
{
  m_fen_id=0;
  m_refTreeModel->clear();
}

void FenList::add_fen(string fen)
{
  Gtk::TreeModel::Row row = *(m_refTreeModel->append());
  row[m_Columns.m_col_id] = ++m_fen_id;
  row[m_Columns.m_col_fen] = fen;
}

int FenList::get_selection(string filename)
{
  clear();
  ifstream ifs(filename);
  string sfen;
  while(!ifs.eof()){
    getline(ifs,sfen);
    if(sfen!=""){add_fen(sfen);}
  }
  ifs.close();



  Gtk::Dialog dialog("Choose Position",m_pw,true);
  dialog.set_border_width(20);
  dialog.set_default_size(400,500);

  //Add response buttons the the dialog:
  dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
  dialog.add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);

  //Prepare content area:
  Gtk::Box* pcontent=dialog.get_content_area();
  pcontent->pack_start(m_VBox);
  pcontent->show_all_children();

  //Show the dialog and wait for a user response:
  int result = dialog.run();

  //Handle the response:
  switch(result)
    {
    case(Gtk::RESPONSE_OK):
      {
	auto iter=m_TreeView.get_selection()->get_selected();
	if(iter){
	  auto row=*iter;
	  m_selection=row[m_Columns.m_col_id];
	}
	return m_selection;
      }
    case(Gtk::RESPONSE_CANCEL):
      {
	return 1;
      }
    default:
      {
	return 1;
      }
    }

}
