/*

Copyright (C) 2018 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include"guiengine.h"
#include<thread>
#include<string>
#include<iostream>
#include<iomanip>
#include<fstream>
#include<sstream>
#include<stdexcept>
using std::setw;
using std::istringstream;
using std::ofstream;
using std::ostringstream;
using std::logic_error;
using std::string;
using std::to_string;
using std::thread;
using std::cout;
using std::endl;
using std::flush;

GuiEngine::GuiEngine()
{
}

GuiEngine::~GuiEngine()
{
}

void GuiEngine::connect(int id,LowLevelEngine& lle)
{
  m_engine_id=id;
  m_pengine=&lle;
  m_pcom_engine=&lle.communicator();
  m_engvar=lle.enginevariables();
}

int GuiEngine::get_engine_id()const
{
  return m_engine_id;
}

void GuiEngine::set_log(bool x)
{
  m_log=x;
  m_pengine->set_log(x);
}

bool GuiEngine::good()const
{
  return m_pengine->good();
}

void GuiEngine::set_logstream(ofstream* x)
{
  m_pofs=x;
}

void GuiEngine::set_white(bool x)
{
  m_engvar.set_white(x);
}

bool GuiEngine::white()const
{
  return m_engvar.white();
}

string GuiEngine::myname()const
{
  return m_engvar.myname();
}

void GuiEngine::set_ponder(bool x)
{
  m_ponder=x;
}

void GuiEngine::reset_clock()
{
  m_engvar.reset_clock();
}

void GuiEngine::start_clock()
{
  m_engvar.start_clock();
}

void GuiEngine::move_and_stop_clock()
{
  m_engvar.move_and_stop_clock();
}

bool GuiEngine::uci()const
{
  return m_engvar.uci();
}

void GuiEngine::set_protocol()
{
  int cnt{0};
  
  //--------------------------- setup uci protocol: -----------------
  //
  m_pcom_engine->to_engine("uci");
  while(true){
    string s=m_pcom_engine->from_engine();
    if(s=="uciok"){m_engvar.set_uci(true);m_engvar.set_san(false);break;}
    else {
      istringstream ist(s);
      string s1;
      ist >> s1;
      if(s1=="id"){
	ist >> s1;
	if(s1=="name"){
	  string temp,s2;
	  while(!ist.eof()){
	    ist >> temp;
	    if(!temp.empty()){s2+=temp;s2+=" ";}
	    temp="";
	  }
	  while(s2.back()==' '){s2.pop_back();}
	  m_engvar.set_myname(s2);
	}
	else if(s1=="author"){
	  getline(ist,s1);
	  m_engvar.set_author(s1);
	}
      }
      else if(s1=="option"){
	Option opt(s);
	m_engvar.add_option(opt);
      }
    }
    std::this_thread::sleep_for(std::chrono::milliseconds(5));
    cnt++;if(cnt>100){break;}
  }
  m_pcom_engine->to_engine("isready");
  cnt=0;
  while(true){
    string s=m_pcom_engine->from_engine();
    if(s=="readyok"){return;}
    std::this_thread::sleep_for(std::chrono::milliseconds(5));
    cnt++;if(cnt>200){break;}
  }

  //--------------------------- setup xboard protocol: -----------------
  //
  m_engvar.set_uci(false);
  m_pcom_engine->to_engine("xboard");
  m_pcom_engine->to_engine("protover 2");
  cnt=0;
  while(true){
    string s=m_pcom_engine->from_engine();
    istringstream is1(s);
    string command;
    is1 >> command;
    if(command=="feature"){
      string features,feature_name,feature_value,answer;
      getline(is1,features);
      istringstream iss_features(features);
      char c;
      iss_features.get(c);
      while(!iss_features.eof()){
	feature_name.clear();feature_value.clear();
	while(c==' ' && !iss_features.eof()){iss_features.get(c);}
	while(c!='=' && !iss_features.eof()){
	  feature_name+=c;
	  iss_features.get(c);
	}
	
	iss_features.get(c);
	if(c=='\"' && !iss_features.eof()){
	  iss_features.get(c);
	  while(c!='\"'){
	    feature_value+=c;
	    iss_features.get(c);
	  }
	  iss_features.get(c);
	}
	else{
	  while(c!=' ' && !iss_features.eof()){
	    feature_value+=c;
	    iss_features.get(c);
	  }
	}
	
	if(feature_name=="ping"){
	  if(feature_value=="1"){m_engvar.set_ping(true);}else{m_engvar.set_ping(false);}
	}
	else if(feature_name=="setboard"){
	  if(feature_value=="1"){m_engvar.set_setboard(true);}else{m_engvar.set_setboard(false);}
	}
	else if(feature_name=="playother"){
	  if(feature_value=="1"){m_engvar.set_playother(true);}else{m_engvar.set_playother(false);}
	}
	else if(feature_name=="san"){
	  if(feature_value=="1"){m_engvar.set_san(true);}else{m_engvar.set_san(false);}
	}
	else if(feature_name=="usermove"){
	  if(feature_value=="1"){m_engvar.set_usermove(true);}else{m_engvar.set_usermove(false);}
	}
	else if(feature_name=="time"){
	  if(feature_value=="1"){m_engvar.set_time(true);}else{m_engvar.set_time(false);}
	}
	else if(feature_name=="draw"){
	  if(feature_value=="1"){m_engvar.set_draw(true);}else{m_engvar.set_draw(false);}
	}
	else if(feature_name=="sigint"){
	  if(feature_value=="1"){m_engvar.set_sigint(true);}else{m_engvar.set_sigint(false);}
	}
	else if(feature_name=="sigterm"){
	  if(feature_value=="1"){m_engvar.set_sigterm(true);}else{m_engvar.set_sigterm(false);}
	}
	else if(feature_name=="reuse"){
	  if(feature_value=="1"){m_engvar.set_reuse(true);}else{m_engvar.set_reuse(false);}
	}
	else if(feature_name=="analyze"){
	  if(feature_value=="1"){m_engvar.set_analyze(true);}else{m_engvar.set_analyze(false);}
	}
	else if(feature_name=="myname"){
	  m_engvar.set_myname(feature_value);
	}
	else if(feature_name=="variants"){
	  m_engvar.set_variants(feature_value);
	}
	else if(feature_name=="colors"){
	  if(feature_value=="1"){m_engvar.set_colors(true);}else{m_engvar.set_colors(false);}
	}
	else if(feature_name=="ics"){
	  if(feature_value=="1"){m_engvar.set_ics(true);}else{m_engvar.set_ics(false);}
	}
	else if(feature_name=="name"){
	  if(feature_value=="1"){m_engvar.set_name(true);}else{m_engvar.set_name(false);}
	}
	else if(feature_name=="pause"){
	  if(feature_value=="1"){m_engvar.set_pause(true);}else{m_engvar.set_pause(false);}
	}
	else if(feature_name=="nps"){
	  if(feature_value=="1"){m_engvar.set_nps(true);}else{m_engvar.set_nps(false);}
	}
	else if(feature_name=="debug"){
	  if(feature_value=="1"){m_engvar.set_debug(true);}else{m_engvar.set_debug(false);}
	}
	else if(feature_name=="memory"){
	  if(feature_value=="1"){
	    m_engvar.set_memory(true);
	    Option opt("option name cecp_memory type spin default 128 min 16 max 1024");
	    m_engvar.add_option(opt);
	  }else{
	    m_engvar.set_memory(false);
	  }
	}
	else if(feature_name=="smp"){
	  if(feature_value=="1"){m_engvar.set_smp(true);}else{m_engvar.set_smp(false);}
	}
	else if(feature_name=="egt"){
	  m_engvar.set_egt(feature_value);
	}
	else if(feature_name=="option"){
	  Option opt("option name " + feature_value);
	  m_engvar.add_option(opt);
	}
	else if(feature_name=="done" && feature_value=="1"){
	  m_engvar.set_done(atoi(feature_value.c_str()));
	  if(m_engvar.done()==1){
	    answer="accepted ";answer+=feature_name;
	    m_pcom_engine->to_engine(answer);
	    set_options(get_options());
	    m_pcom_engine->to_engine("new");
	  }
	  return;
	}
	answer="accepted ";answer+=feature_name;
	m_pcom_engine->to_engine(answer);
      }
    }
    std::this_thread::sleep_for(std::chrono::milliseconds(5));
    cnt++;if(cnt>100){break;}      //throw logic_error("setup xboard protocol failed");}
  }
}

void GuiEngine::set_options(Map x)
{
  m_engvar.set_options(x);
  for(auto v : x){
    string s;
    if(m_engvar.uci()){
      s="setoption name ";
      s+=v.second.name();
      if(v.second.value()!=""){s+=" value "+v.second.value();}
    }else{
      s="option ";
      s+=v.second.name();
      string sv=v.second.value();

      //--------------------- special CECP case: memory   (cores not implemented) -------------
      if(v.second.name()=="cecp_memory"){
	s="memory "+sv;
	m_pcom_engine->to_engine(s);
	continue;
      }
      //----------------------------------------------------------------------------------

      if(sv=="true"){sv="1";}else if(sv=="false"){sv="0";}
      if(sv!=""){s+="="+ sv;}
    }

    m_pcom_engine->to_engine(s);
  }
}

Map GuiEngine::get_options()
{
  return m_engvar.get_options();
}

void GuiEngine::set_opponent(GuiEngine* opp)
{
  m_opponent=opp;
}

void GuiEngine::position(const ChessPosition& cp)
{
  m_cp=cp;
}

void GuiEngine::newgame()
{
  m_cp.newgame();
  reset_clock();
}

string GuiEngine::get_answer()
{
  string r{"GuiEngine::get_answer() error"};
  return r;
}

void GuiEngine::set_level(const Level& x)
{
  m_engvar.set_level(x);
}

Duration GuiEngine::get_time()   
{
  return m_engvar.get_time();
}

Duration GuiEngine::get_wtime()   
{
  if(white()){return get_time();}
  else{return m_opponent->get_time();}
}

Duration GuiEngine::get_btime()   
{
  if(!white()){return get_time();}
  else{return m_opponent->get_time();}
}

Duration GuiEngine::get_inc()   
{
  return m_engvar.get_inc();
}

int GuiEngine::get_movestogo()   
{
  return m_engvar.get_movestogo();
}

Move GuiEngine::alg2move(string s)
{
  return m_cp.convert_to_move(s);
}

void GuiEngine::go()
{
}

void GuiEngine::go_infinite()
{
}

void GuiEngine::stop()
{
}
