/*

Copyright (C) 2018 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#ifndef GUIENGINE_H
#define GUIENGINE_H

#include<string>
#include<thread>
#include"../engine/board/cposition.h"
#include"../engine/communicator.h"
#include"lowlevelengine.h"
#include"pgnfile.h"
#include"engio.h"
#include"engvar.h"
#include"level.h"
using std::string;
using std::thread;
using Map=std::map<string,Option>;

class CecpEngine;
class UciEngine;

class GuiEngine
{
 private:
  void set_protocol();
  
 protected:
  int m_engine_id{0};
  LowLevelEngine* m_pengine{nullptr};
  EngineVariables m_engvar;
  Communicator* m_pcom_engine;

  bool m_ponder{false};
  GuiEngine* m_opponent;
  ChessPosition m_cp;
  bool m_log{false};
  ofstream* m_pofs{nullptr};

 public:
  GuiEngine();
  GuiEngine(const GuiEngine&)=delete;
  GuiEngine& operator=(const GuiEngine&)=delete;
  GuiEngine(GuiEngine&&)=delete;
  GuiEngine& operator=(GuiEngine&&)=delete;
  virtual ~GuiEngine();

  void connect(int,LowLevelEngine&);
  void set_options(Map);
  void set_log(bool);
  void set_logstream(ofstream*);
  void set_white(bool);
  void reset_clock();
  void start_clock();
  void move_and_stop_clock();
  void position(const ChessPosition&);
  void set_level(const Level&);
  void set_opponent(GuiEngine*);
  bool uci()const;
  bool good()const;
  bool white()const;
  Map get_options();
  int get_engine_id()const;
  Duration get_time();
  Duration get_wtime();
  Duration get_btime();
  Duration get_inc();
  int get_movestogo();
  string myname()const;
  Move alg2move(string);
  virtual void set_ponder(bool);
  virtual void newgame();
  virtual void go();
  virtual void go_infinite();
  virtual void stop();
  virtual string get_answer();
};


#endif
