/*

Copyright (C) 2018 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#ifndef INTERFACE_H
#define INTERFACE_H

#include<gtkmm.h>
#include<functional>
#include<thread>
#include<tuple>
#include<map>
#include"board.h"
#include"clocks.h"
#include"master.h"
#include"../engine/board/cposition.h"
#include"../engine/communicator.h"
#include"pgnfile.h"
#include"game.h"
#include"ecocodes.h"
#include"optionsdialog.h"
#include"anaw.h"
#include"epdw.h"
#include"gamelist.h"
#include"fenlist.h"
#include"pgnview.h"
#include"commenteditor.h"
using std::tuple;


class Interface : public Gtk::Window
{
 private:
  // master:
  Master master;
  thread m_t1;

  // Functions:
  string file_chooser(const char);
  tuple<int,int,string,float> time_control();
  void telluser(string,string);

  // Signal handlers:
  bool on_piece_moved();
  bool on_timeout(int);
  void on_opening_book();
  void on_menu_new_game();
  void on_menu_load_position();
  void on_menu_load_next_position();
  void on_menu_load_previous_position();
  void on_menu_save_position();
  void on_menu_load_game();
  void on_menu_save_game();
  void on_menu_setup();
  void on_menu_quit();
  void on_menu_backward();
  void on_menu_forward();
  void on_menu_back_to_start();
  void on_menu_forward_to_end();
  void on_menu_human_human();
  void on_menu_human_computer();
  void on_menu_analyze_position();
  void on_menu_start_thinking();
  void on_menu_computer_computer();
  void on_menu_tournament();
  void on_menu_flip();
  void on_menu_show_output();
  void on_menu_show_pgnviewer();
  void on_menu_show_commenteditor();
  void on_menu_show_epd_commands();
  void on_menu_engine1();
  void on_menu_engine2();
  void on_menu_about();
  void on_menu_timecontrol();
  void on_menu_engine1_options();
  void on_menu_engine2_options();
  void on_menu_ponder();
  void on_menu_debug();
  bool on_delete_event(GdkEventAny*);
  
  // Child widgets:
  Gtk::Box m_Box;
  Glib::RefPtr<Gtk::UIManager> m_refUIManager;
  Glib::RefPtr<Gtk::ActionGroup> m_refActionGroup;
  Glib::RefPtr<Gtk::RadioAction> m_setup, m_human_human, m_human_computer, m_computer_computer, m_tournament, m_analyze_position;
  Glib::RefPtr<Gtk::ToggleAction> m_ponder;
  
  Gtk::ButtonBox m_navigation_box;
  Gtk::Button m_goto_first,m_go_backward,m_go_forward,m_goto_last,m_book;
  Gtk::Image m_image_goto_first,m_image_go_backward,m_image_go_forward,m_image_goto_last;

  // other windows:
  Board m_board;
  Clocks m_clocks;
  AnalysisWindow m_analysis_window_engine1,m_analysis_window_engine2;
  PgnView m_pgnview;
  CommentEditor m_commenteditor;
  EpdCommandsWindow m_epd_commands_window;

  // load games and fens:
  GameList m_gamelist;
  FenList m_fenlist;
  string m_fen_filename;
  string m_fen;
  int m_chosen_fen{0};
  EcoCodes m_eco;
  
  // state variables:
  ChessPosition cp;
  enum class Player{human,machine1,machine2};
  Player m_white,m_black;
  string m_path_engine1,m_path_engine2;
  string m_name_whiteplayer,m_name_blackplayer,m_name_engine1,m_name_engine2;
  int m_round_number{1};
  string m_white_comment,m_black_comment;
  Timer m_last_click_timer;
  string m_level{"40 1:0 0.0"};
  
  
  Communicator& com;
  Game m_game;
  bool m_setup_position{false};
  bool m_debug{false};
  bool m_show_engine_output{false};
  bool m_show_pgnview{false};
  bool m_show_commenteditor{false};
  bool m_show_epd_commands{false};
  bool m_force{false};
  void draw_board();
  void get_master_command();
  string m_prefix;
  string m_sound;
  string m_playsound;
  
  enum {OpenFen,SaveFen,OpenPgn,OpenPgnEpd,SavePgn,OpenExe};


 public:
  Interface(Communicator&,string);
  virtual ~Interface();
};



#endif
