/*

Copyright (C) 2018 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#ifndef MASTER_H
#define MASTER_H

#include<string>
#include<chrono>
#include"../engine/board/cposition.h"
#include"../engine/communicator.h"
#include"../engine/search/timer.h"
#include"guiengine.h"
#include"uci_engine.h"
#include"cecp_engine.h"
#include"lowlevelengine.h"
#include"pgnfile.h"
#include"engio.h"
#include"engvar.h"
#include"level.h"
using namespace std::chrono;
using std::milli;
using dura=duration<double,milli>;
using std::string;
using Map=std::map<string,Option>;

class Master
{
 private:
  bool debug{false};
  Level m_level;
  bool m_log;
  bool m_ponder{false};
  bool m_accept_engine_time{false};
  ofstream m_ofs;
  string m_guiname;
  Communicator& m_com_interface;
  ChessPosition m_cp;
  bool m_quit;

  //----- engines: 
  GuiEngine *m_peng1,*m_peng2;
  UciEngine m_uci_eng1,m_uci_eng2;
  CecpEngine m_cecp_eng1,m_cecp_eng2;
  LowLevelEngine m_lle1,m_lle2;
  
  
  //----- state of the master: -----------------------------------------
  enum class MasterStatus{human_computer,human_human,match,round_robin,swiss,gauntlet,keyzer};
  enum class MatchStatus{idle,human_computer,computer_computer};
  enum class GameStatus{idle,human_to_move,engine1_to_move,engine2_to_move};  
  MasterStatus m_status{MasterStatus::human_computer};
  MatchStatus m_match_status{MatchStatus::human_computer};
  GameStatus m_game_status{GameStatus::human_to_move};

  Move m_unprocessed_move;
  int m_engine_to_move{1};             // 0: human to move   1: first engine to move    2: second engine to move
  //--------------------------------------------------------------------

  
  //----- match parameters:
  string m_bookname;
  PgnFile m_book;
  int m_number_of_rounds;
  int m_current_round;
  string m_current_opening;
  int m_engine1_wins;
  int m_engine2_wins;
  int m_draws;

  //----- private functions:
  Move alg2move(string);
  void setup_bookposition();
  void end_of_game_processing(string="");
  void master_directions();
  void process_command_from_interface();
  void newgame();
  void load_galjoen();
  void load_engine(int,string);
  void reload_engine(int);
  template<int I> void process_answer_from_engine();
  template<int I> void send_info(string);

 public:
  Master(Communicator&,string);
  ~Master();
  void operator()();
  Map get_options_engine1();
  Map get_options_engine2();
  void set_log(bool);
  void dump();
};




#endif
