/*

Copyright (C) 2018 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include"uci_engine.h"
#include"engvar.h"
#include<stdexcept>
#include<iostream>
#include<sstream>
using std::logic_error;
using std::endl;
using std::istringstream;
using std::ostringstream;


UciEngine::~UciEngine()
{
}

void UciEngine::newgame()
{
  stop();
  reset_clock();
  m_cp.newgame();
  m_pcom_engine->to_engine("ucinewgame");
  m_pcom_engine->to_engine("isready");
  int cnt{0};
  while(true){
    string s=m_pcom_engine->from_engine();
    if(s=="readyok"){return;}
    std::this_thread::sleep_for(std::chrono::milliseconds(5));
    cnt++;if(cnt>100){break;}
  }
  throw logic_error(R"(UciEngine::newgame()   engine doesn't respond to isready)");
}

void UciEngine::stop()
{
  if(m_engvar.get_status()!=EngineStatus::thinking && m_engvar.get_status()!=EngineStatus::pondering){return;}
  m_pcom_engine->to_engine("stop");
  if(m_log){*m_pofs << "engine" << m_engine_id << " <- UciEngine::stop()" << endl;}
  string s;
  for(int i=0;i<1000;i++){
    s=m_pcom_engine->from_engine();
    istringstream iss(s);
    string bm;iss >> bm;
    if(bm=="bestmove"){
      break;
    }
    std::this_thread::sleep_for(milliseconds(5));
    if(i==999){if(m_log){*m_pofs << "engine" << m_engine_id << "  UciEngine::stop() engine doesn't respond with bestmove" << endl;}break;}
  }
  m_engvar.set_status(EngineStatus::idle);
}

void UciEngine::go()
{
  if(m_engvar.get_status()==EngineStatus::thinking){
    throw logic_error("UciEngine::go():   engine already thinking");
  }
  if(m_engvar.get_status()==EngineStatus::pondering){
    string pondermove=m_cp.history().back().last_move.algebraic();
    if(pondermove==m_pondermove){
      m_pcom_engine->to_engine("ponderhit");
      if(m_log){*m_pofs << "engine" << m_engine_id << " <- UciEngine::go()    ponderhit" << endl;}
      start_clock();
      m_engvar.set_status(EngineStatus::thinking);
      return;
    }
    m_pcom_engine->to_engine("stop");
    if(m_log){*m_pofs << "engine" << m_engine_id << " <- UciEngine::go()    stop" << endl;}
    string s;
    for(int i=0;i<1000;i++){
      s=m_pcom_engine->from_engine();
      istringstream iss(s);
      string bm;iss >> bm;
      if(bm=="bestmove"){
	m_engvar.set_status(EngineStatus::idle);
	break;
      }
      std::this_thread::sleep_for(milliseconds(5));
      if(i==999){throw logic_error("UciEngine keeps pondering");}
    }    
  }
  
  string s="position fen ";
  s+=m_cp.initial_position();
  s+=" moves ";
  for(size_t i=1;i<m_cp.history().size();i++){s+=m_cp.history()[i].last_move.algebraic();s+=" ";}
  while(s.back()==' '){s.pop_back();}
  m_pcom_engine->to_engine(s);
  m_pcom_engine->to_engine("isready");
  s.clear();
  for(int i=0;i<1000;i++){
    s=m_pcom_engine->from_engine();
    if(s=="readyok"){break;}
    std::this_thread::sleep_for(milliseconds(5));
    if(i==999){throw logic_error("UciEngine  does not respond");}
  }

  int wtime,btime,winc,binc,movestogo;
  if(white()){
    wtime=get_time().milliseconds();
    winc=get_inc().milliseconds();
    btime=m_opponent->get_time().milliseconds();
    binc=m_opponent->get_inc().milliseconds();
    movestogo=get_movestogo();
  }else{
    wtime=m_opponent->get_time().milliseconds();
    winc=m_opponent->get_inc().milliseconds();
    btime=get_time().milliseconds();
    binc=get_inc().milliseconds();
    movestogo=get_movestogo();
  }
  ostringstream oss;
  oss << "go wtime " << wtime << " btime " << btime
      << " winc " << winc << " binc " << binc;
  if(movestogo>0){oss << "  movestogo " << movestogo;}
  s=oss.str();
  m_pcom_engine->to_engine(s);
  start_clock();
  m_engvar.set_status(EngineStatus::thinking);
  if(m_log){*m_pofs << "engine" << m_engine_id << " <- UciEngine::go()    " << s << endl;}
}

void UciEngine::go_infinite()
{
  m_pcom_engine->to_engine("stop");
  if(m_log){*m_pofs << "engine" << m_engine_id << " <- UciEngine::go()    stop" << endl;}
  string s;
  for(int i=0;i<1000;i++){
    s=m_pcom_engine->from_engine();
    istringstream iss(s);
    string bm;iss >> bm;
    if(bm=="bestmove"){
      m_engvar.set_status(EngineStatus::idle);
      break;
    }
    std::this_thread::sleep_for(milliseconds(5));
    if(i==999){throw logic_error("UciEngine keeps pondering");}
  }    
 
  s="position fen ";
  s+=m_cp.initial_position();
  s+=" moves ";
  for(size_t i=1;i<m_cp.history().size();i++){s+=m_cp.history()[i].last_move.algebraic();s+=" ";}
  while(s.back()==' '){s.pop_back();}
  m_pcom_engine->to_engine(s);
  m_pcom_engine->to_engine("isready");
  s.clear();
  for(int i=0;i<1000;i++){
    s=m_pcom_engine->from_engine();
    if(s=="readyok"){break;}
    std::this_thread::sleep_for(milliseconds(5));
    if(i==999){throw logic_error("UciEngine  does not respond");}
  }
  ostringstream oss;
  oss << "go infinite";
  s=oss.str();
  m_pcom_engine->to_engine(s);
  m_engvar.set_status(EngineStatus::thinking);
  if(m_log){*m_pofs << "engine" << m_engine_id << " <- UciEngine::go()    " << s << endl;}
}

void UciEngine::goponder()
{
  if(m_engvar.get_status()!=EngineStatus::idle){
    throw logic_error("UciEngine::goponder() while engine not idle");
  }
  string s="position fen ";
  s+=m_cp.initial_position();
  s+=" moves ";
  for(size_t i=1;i<m_cp.history().size();i++){s+=m_cp.history()[i].last_move.algebraic();s+=" ";}
  while(s.back()==' '){s.pop_back();}
  m_pcom_engine->to_engine(s);
  m_pcom_engine->to_engine("isready");
  s.clear();
  for(int i=0;i<1000;i++){
    s=m_pcom_engine->from_engine();
    if(s=="readyok"){break;}
    std::this_thread::sleep_for(milliseconds(5));
    if(i==999){throw logic_error("UciEngine  does not respond");}
  }

  int wtime,btime,winc,binc,movestogo;
  if(white()){
    wtime=get_time().milliseconds();
    winc=get_inc().milliseconds();
    btime=m_opponent->get_time().milliseconds();
    binc=m_opponent->get_inc().milliseconds();
    movestogo=get_movestogo();
  }else{
    wtime=m_opponent->get_time().milliseconds();
    winc=m_opponent->get_inc().milliseconds();
    btime=get_time().milliseconds();
    binc=get_inc().milliseconds();
    movestogo=get_movestogo();
  }
  ostringstream oss;
  oss << "go wtime " << wtime << " btime " << btime
      << " winc " << winc << " binc " << binc;
  if(movestogo>0){oss << "  movestogo " << movestogo;}
  oss << " ponder";
  s=oss.str();
  m_pcom_engine->to_engine(s);
  m_engvar.set_status(EngineStatus::pondering);
}

string UciEngine::get_answer()
{
  string move;
  string answer=m_pcom_engine->from_engine();
  if(answer.size()>0){
    istringstream iss(answer);
    string s;iss >> s;
    if(s=="bestmove"){
      m_engvar.set_status(EngineStatus::idle);
      m_pondermove="";
      move_and_stop_clock();
      iss >> move;
    }
    if(!iss.eof()){
      iss >> s;
      if(s=="ponder" && m_ponder){
	iss >> m_pondermove;
	Move m=alg2move(move);
	m_cp.make_move(m);
	m=alg2move(m_pondermove);
	m_cp.make_move(m);
	goponder();
      }
    }
  }

  return answer;
}

